from matplotlib import animation
from clawpack.visclaw.JSAnimation import IPython_display
from IPython.display import display
import ipywidgets
import sympy
import numpy as np
import matplotlib.pyplot as plt
import matplotlib


sympy.init_printing(use_latex='mathjax')


def riemann_solution(solver,q_l,q_r,aux_l=None,aux_r=None,t=0.2,problem_data=None,verbose=False):
    r"""
    Compute the (approximate) solution of the Riemann problem with states (q_l, q_r)
    based on the (approximate) Riemann solver `solver`.  The solver should be
    a pointwise solver provided as a Python function.

    **Example**::

        # Call solver
        >>> from clawpack import riemann
        >>> gamma = 1.4
        >>> problem_data = { 'gamma' : gamma, 'gamma1' : gamma - 1 }
        >>> solver = riemann.euler_1D_py.euler_hll_1D
        >>> q_l = (3., -0.5, 2.); q_r = (1., 0., 1.)
        >>> states, speeds, reval = riemann_solution(solver, q_l, q_r, problem_data=problem_data)

        # Check output
        >>> q_m = np.array([1.686068, 0.053321, 1.202282])
        >>> assert np.allclose(q_m, states[:,1])

    """
    if not isinstance(q_l, np.ndarray):
        q_l = np.array(q_l)   # in case q_l, q_r specified as scalars
        q_r = np.array(q_r)

    num_eqn = len(q_l)

    if aux_l is None:
        aux_l = np.zeros(num_eqn)
    if aux_r is None:
        aux_r = np.zeros(num_eqn)

    wave, s, amdq, apdq = solver(q_l.reshape((num_eqn,1)),q_r.reshape((num_eqn,1)),
                                 aux_l.reshape((num_eqn,1)),aux_r.reshape((num_eqn,1)),problem_data)
    
    wave0 = wave[:,:,0]
    num_waves = wave.shape[1]
    qlwave = np.vstack((q_l,wave0.T)).T
    # Sum to the waves to get the states:
    states = np.cumsum(qlwave,1)  
    
    num_states = num_waves + 1
    
    if verbose:
        print('States in Riemann solution:')
        states_sym = sympy.Matrix(states)
        display([states_sym[:,k] for k in range(num_states)])
    
        print('Waves (jumps between states):')
        wave_sym = sympy.Matrix(wave[:,:,0])
        display([wave_sym[:,k] for k in range(num_waves)])
    
        print("Speeds: ")
        s_sym = sympy.Matrix(s)
        display(s_sym.T)
    
        print("amdq, apdq: ")
        amdq_sym = sympy.Matrix(amdq).T
        apdq_sym = sympy.Matrix(apdq).T
        display([amdq_sym, apdq_sym])
    
    def riemann_eval(xi):
        "Return Riemann solution as function of xi = x/t."
        qout = np.zeros((num_eqn,len(xi)))
        intervals = [(xi>=s[i])*(xi<=s[i+1]) for i in range(len(s)-1)]
        intervals.insert(0, xi<s[0])
        intervals.append(xi>=s[-1])
        for m in range(num_eqn):
            qout[m,:] = np.piecewise(xi, intervals, states[m,:])

        return qout

    return states, s, riemann_eval

def plot_phase(states, i_h=0, i_v=1, ax=None):
    """
    Plot 2d phase space plot.
    If num_eqns > 2, can specify which component of q to put on horizontal
    and vertical axes via i_h and i_v.
    """
    q0 = states[i_h,:]
    q1 = states[i_v,:]
    
    if ax is None:
        fig, ax = plt.subplots()
    ax.plot(q0,q1,'o-k')
    ax.set_title('phase space: q[%i] vs. q[%i]' % (i_h,i_v))
    ax.axis('equal')
    dq0 = q0.max() - q0.min()
    dq1 = q1.max() - q1.min()
    ax.text(q0[0] + 0.05*dq0,q1[0] + 0.05*dq1,'q_left')
    ax.text(q0[-1] + 0.05*dq0,q1[-1] + 0.05*dq1,'q_right')
    ax.axis([q0.min()-0.1*dq0, q0.max()+0.1*dq0, q1.min()-0.1*dq1, q1.max()+0.1*dq1])
    ax.set_xlabel('q[%s]' % i_h)
    ax.set_ylabel('q[%s]' % i_v)
    
def plot_phase_3d(states):
    """
    3d phase space plot
    """
    from mpl_toolkits.mplot3d import Axes3D
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.plot(states[0,:],states[1,:],states[2,:],'ko-')
    ax.set_xlabel('q[0]')
    ax.set_ylabel('q[1]')
    ax.set_zlabel('q[2]')
    ax.set_title('phase space')
    ax.text(states[0,0]+0.05,states[1,0],states[2,0],'q_left')
    ax.text(states[0,-1]+0.05,states[1,-1],states[2,-1],'q_right')
 
def plot_riemann(states, s, riemann_eval, t, fig=None, color='b', layout='horizontal',conserved_variables=None):
    """
    Take an array of states and speeds s and plot the solution at time t.
    For rarefaction waves, the corresponding entry in s should be tuple of two values,
    which are the wave speeds that bound the rarefaction fan.

    Plots in the x-t plane and also produces a separate plot for each component of q.
    """
    
    num_eqn,num_states = states.shape
    if fig is None:
        if layout == 'horizontal':
            fig_width = 4*(num_eqn+1)
            fig, ax = plt.subplots(1,num_eqn+1,figsize=(fig_width,4))
        elif layout == 'vertical':
            fig_width = 9
            fig_height = 4*num_eqn
            fig, ax = plt.subplots(num_eqn+1,1,figsize=(fig_width,fig_height),sharex=True)
            plt.subplots_adjust(hspace=0)
            ax[-1].set_xlabel('x')
            ax[0].set_ylabel('t')
            ax[0].set_title('t = %6.3f' % t)
    else:
        ax = fig.axes
        xmin, xmax = ax[1].get_xlim()

    for axis in ax:
        for child in axis.get_children():
            if isinstance(child, matplotlib.spines.Spine):
                child.set_color('#dddddd')

    tmax = 1.0
    xmax = 0.
    for i in range(len(s)):
        if type(s[i]) not in (tuple, list):  # this is a jump
            x1 = tmax * s[i]
            ax[0].plot([0,x1],[0,tmax],color=color)
            xmax = max(xmax,abs(x1))
        else: #plot rarefaction fan
            speeds = np.linspace(s[i][0],s[i][1],5)
            for ss in speeds:
                x1 = tmax * ss
                ax[0].plot([0,x1],[0,tmax],color=color,lw=0.3)
                xmax = max(xmax,abs(x1))

    x = np.linspace(-xmax,xmax,1000)
                   
    ax[0].set_xlim(-xmax,xmax)
    ax[0].plot([-xmax,xmax],[t,t],'k',linewidth=2)
    

    if conserved_variables is None:
        conserved_variables = ['q[%s]' % i for i in range(num_eqn)]

    for i in range(num_eqn):
        ax[i+1].set_xlim((-1,1))
        qmax = states[i,:].max()  #max([state[i] for state in states])
        qmin = states[i,:].min()  # min([state[i] for state in states])
        qdiff = qmax - qmin
        ax[i+1].set_xlim(-xmax,xmax)
        ax[i+1].set_ylim((qmin-0.1*qdiff,qmax+0.1*qdiff))
        if layout == 'horizontal':
            ax[i+1].set_title(conserved_variables[i]+' at t = %6.3f' % t)
        elif layout == 'vertical':
            ax[i+1].set_ylabel(conserved_variables[i])
    
    if t == 0:
        q = riemann_eval(x/1e-10)
    else:
        q = riemann_eval(x/t)

    for i in range(num_eqn):
        ax[i+1].plot(x,q[i][:],color=color,lw=2)

    return fig
            

def make_plot_function(states_list,speeds_list,riemann_eval_list,names=None,layout='horizontal',conserved_variables=None):
    """
    Utility function to create a plot_function that takes a single argument t.
    This function can then be used in a StaticInteract widget.
    Version that takes an arbitrary list of sets of states and speeds in order to make a comparison.
    """
    colors = "kbrg"
    if type(states_list) is not list:
        states_list = [states_list]
        speeds_list = [speeds_list]
        riemann_eval_list = [riemann_eval_list]

    def plot_function(t):
        fig = None
        for i in range(len(states_list)):
            states = states_list[i]
            speeds = speeds_list[i]
            riemann_eval = riemann_eval_list[i]

            if fig is None:
                fig = plot_riemann(states,speeds,riemann_eval,t,color=colors[i],layout=layout,conserved_variables=conserved_variables)
            else:
                fig = plot_riemann(states,speeds,riemann_eval,t,fig,colors[i],layout=layout,conserved_variables=conserved_variables)

            if names is not None:
                # We could use fig.legend here if we had the line plot handles
                fig.axes[1].legend(names,loc='best')

        return fig

    return plot_function


if __name__ == '__main__':
    import doctest
    doctest.testmod()



import os
from itertools import chain
import numpy as np
from clawpack.pyclaw.util import check_diff
from . import acoustics_3d_interface

def check_error(**kwargs):

    claw = acoustics_3d_interface.setup(problem='homogeneous',mx=128,my=4,mz=4,disable_output=True,**kwargs)
    claw.run()

    test_pressure = claw.frames[-1].q[0,:,:,:]
    expected_pressure = claw.frames[0].q[0,:,:,:]
    return check_diff(expected_pressure, test_pressure, reltol=1e-1,
                        delta=claw.solution.grid.delta)


class TestAcoustics3D:
    def test_classic(self):
        assert check_error(solver_type='classic')==None

    def test_sharpclaw(self):
        assert check_error(solver_type='sharpclaw')==None



#!/usr/bin/env python

r"""chile2010_adjoint regression test for GeoClaw

To create new regression data use
    `python regression_tests.py True`
"""

from __future__ import absolute_import
import os
import sys
import unittest
import shutil

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools
from clawpack.clawutil.test import wip


try:
    CLAW = os.environ['CLAW']
except:
    raise Exception("*** Must first set CLAW enviornment variable")

# Scratch directory for storing topo and dtopo files:
scratch_dir = os.path.join(CLAW, 'geoclaw', 'scratch')

thisfile = os.path.realpath(__file__)
testdir = os.path.split(thisfile)[0]


class Chile2010AdjointTest(test.GeoClawRegressionTest):

    r"""Chile2010AdjointTest regression test for GeoClaw"""

    def setUp(self):

        super(Chile2010AdjointTest, self).setUp()

        start_dir = os.getcwd()
        test_adjoint_path = os.path.join(self.test_path, 'adjoint')
        temp_adjoint_path = os.path.join(self.temp_path, 'adjoint')
        #print('+++ test_adjoint_path = ',test_adjoint_path)
        #print('+++ temp_adjoint_path = ',temp_adjoint_path)

        shutil.copytree(test_adjoint_path, temp_adjoint_path)

        # run adjoint code
        os.chdir(temp_adjoint_path)
        #print('+++ Running adjoint in directory ',os.getcwd())
        #print('+++   contents: ', os.listdir('.'))
        os.system('make -s topo')
        os.system('make -s data')
        os.system('make -s new')
        os.system('make .output > output.txt')
        #print('+++   contents of _output: ', os.listdir('_output'))

        # set up forward code
        shutil.copy(os.path.join(self.test_path, "maketopo.py"),
                                 self.temp_path)
        os.chdir(self.temp_path)
        #print('+++ Running forward in directory ',os.getcwd())
        #print('+++   contents: ', os.listdir())
        os.system('python maketopo.py')
        #print('+++   scratch directory: ', scratch_dir)
        #print('+++   contents of scratch: ', os.listdir(scratch_dir))
        os.chdir(start_dir)


    def runTest(self, save=False, indices=(2, 3)):
        r"""Test chile2010_adjoint example

        Note that this stub really only runs the code and performs no tests.

        """

        # Write out data files
        self.load_rundata()
        temp_adjoint_path = os.path.join(self.temp_path, 'adjoint')
        self.rundata.adjointdata.adjoint_outdir = \
             os.path.join(temp_adjoint_path, '_output')
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, indices=(2, 3))
        self.success = True


if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = Chile2010AdjointTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
    
from __future__ import absolute_import
def acoustics(problem='figure 9.4'):
    """
    This example solves the 1-dimensional variable-coefficient acoustics
    equations in a medium with a single interface.
    """
    from numpy import sqrt, abs
    from clawpack import pyclaw
    from clawpack import riemann

    solver = pyclaw.ClawSolver1D(riemann.acoustics_variable_1D)

    solver.limiters = pyclaw.limiters.tvd.MC
    solver.bc_lower[0] = pyclaw.BC.extrap
    solver.bc_upper[0] = pyclaw.BC.extrap
    solver.aux_bc_lower[0] = pyclaw.BC.extrap
    solver.aux_bc_upper[0] = pyclaw.BC.extrap

    x = pyclaw.Dimension(-5.0,5.0,500,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 2
    num_aux = 2
    state = pyclaw.State(domain, num_eqn, num_aux)

    if problem == 'figure 9.4':
        rhol = 1.0
        cl   = 1.0
        rhor = 2.0
        cr   = 0.5
    elif problem == 'figure 9.5':
        rhol = 1.0
        cl   = 1.0
        rhor = 4.0
        cr   = 0.5
    zl = rhol*cl
    zr = rhor*cr
    xc = domain.grid.x.centers

    state.aux[0,:] = (xc<=0)*zl + (xc>0)*zr  # Impedance
    state.aux[1,:] = (xc<=0)*cl + (xc>0)*cr  # Sound speed

    # initial condition: half-ellipse
    state.q[0,:] = sqrt(abs(1.-(xc+3.)**2))*(xc>-4.)*(xc<-2.)
    state.q[1,:] = state.q[0,:] + 0.

    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.tfinal = 5.0
    claw.num_output_times   = 10

    # Solve
    return claw

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(acoustics)



"""
Make figures 2 and 3 for journal article:

    Shoaling on Steep Continental Slopes: Relating Transmission and 
    Reflection Coefficients to Green's Law, 
    by J. George, D. I. Ketcheson, and R. J. LeVeque,
    Pure and Applied Geophysics, 2019.
    DOI 10.1007/s00024-019-02316-y

See http://faculty.washington.edu/rjl/pubs/Shoaling2019 for additional links.

"""
from pylab import *
import os

from clawpack.clawutil.runclaw import runclaw
from clawpack.visclaw.frametools import plotframe

from clawpack.visclaw.data import ClawPlotData
import make_celledges
from imp import reload
reload(make_celledges)

run_code = False  # set to False if output already exists
if run_code:
    # create executable and .data files:
    os.system('make .exe')
    os.system('make data')

savefig_ext = '.png'
figdir = './figures'
os.system('mkdir -p %s' % figdir)

def save_figure(fname):
    """Save figure to figdir with desired extension"""
    full_fname = os.path.join(figdir,fname) + savefig_ext
    savefig(full_fname, bbox_inches='tight')
    print('Created %s' % full_fname)

for xs in [0., 15e3, 60e3]:
    make_celledges.makegrid(xs)
    close('all')
    #import mapc2p
    import setplot
    #reload(mapc2p)
    reload(setplot)
    xs_str = 'box_xs%s' % str(int(xs/1e3)).zfill(2)
    w_str = 'w%s' % str(int(xs/1e3)).zfill(2)
    outdir = '_output_' + w_str
    print('outdir = ',outdir)
    if run_code:
        runclaw(xclawcmd='xgeo',outdir=outdir)   # run clawpack code
    pd = ClawPlotData()
    pd.outdir = outdir
    pd = setplot.setplot(pd)  # reload celledges.data for each xs value
    pd.outdir = os.path.abspath(outdir)
    pd.printfigs = False  # or else figure is closed after printing
    for frameno in [0,6]:
        plotframe(frameno,pd)
        fname = xs_str + '_frame%s' % str(frameno).zfill(2)
        save_figure(fname)





#!/usr/bin/env python
# encoding: utf-8
r"""
Module containing SharpClaw solvers for PetClaw
"""

from __future__ import absolute_import
from clawpack import pyclaw

class SharpClawSolver1D(pyclaw.SharpClawSolver1D):
    """1D parallel SharpClaw solver.
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw.SharpClawSolver2D)

    
class SharpClawSolver2D(pyclaw.SharpClawSolver2D):
    """2D parallel SharpClaw solver. 
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw.SharpClawSolver2D)


class SharpClawSolver3D(pyclaw.SharpClawSolver3D):
    """3D parallel SharpClaw solver. 
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw.SharpClawSolver3D)



#!/usr/bin/env python
# encoding: utf-8
"""
Wave propagation Riemann solvers implemented in Python and Fortran.
"""

rp_solver_list_1d = []
rp_solver_list_2d = []
rp_solver_list_3d = []

# Import 1d Riemann solvers
from . import advection_1D_py
from . import vc_advection_1D_py
from . import acoustics_1D_py
from . import burgers_1D_py
from . import shallow_1D_py
from . import euler_1D_py
from . import nonlinear_elasticity_1D_py
from . import static

from . import acoustics_1D_constants
from . import acoustics_variable_1D_constants
from . import advection_1D_constants
from . import burgers_1D_constants
from . import cubic_1D_constants
from . import euler_with_efix_1D_constants
from . import mhd_1D_constants
from . import nonlinear_elasticity_fwave_1D_constants
from . import reactive_euler_with_efix_1D_constants
from . import shallow_roe_with_efix_1D_constants
from . import shallow_roe_tracer_1D_constants
from . import traffic_1D_constants
from . import traffic_vc_1D_constants
from . import traffic_vc_tracer_1D_constants
from . import acoustics_2D_constants
from . import acoustics_mapped_2D_constants
from . import advection_2D_constants
from . import burgers_2D_constants
from . import euler_mapgrid_2D_constants
from . import euler_5wave_2D_constants
from . import euler_4wave_2D_constants
from . import kpp_2D_constants
from . import psystem_2D_constants
from . import shallow_roe_with_efix_2D_constants
from . import shallow_sphere_2D_constants
from . import vc_acoustics_2D_constants
from . import vc_advection_2D_constants
from . import vc_elasticity_2D_constants
from . import vc_acoustics_3D_constants
from . import euler_3D_constants
from . import burgers_3D_constants
from . import vc_advection_3D_constants
from . import euler_mapgrid_3D


try:
    from . import acoustics_1D
    from . import acoustics_variable_1D
    from . import acoustics_1D_ptwise
    from . import advection_1D
    from . import advection_1D_ptwise
    from . import advection_color_1D
    from . import burgers_1D
    from . import cubic_1D
    from . import euler_with_efix_1D
    from . import euler_hlle_1D
    from . import mhd_roe_1D
    from . import nonlinear_elasticity_fwave_1D
    from . import psystem_fwave_1D
    from . import reactive_euler_with_efix_1D
    from . import shallow_hlle_1D
    from . import shallow_roe_with_efix_1D
    from . import shallow_bathymetry_fwave_1D
    from . import shallow_roe_tracer_1D
    from . import traffic_1D
    from . import traffic_vc_1D
    from . import traffic_vc_fwave_1D
    from . import traffic_vc_tracer_1D
    from . import acoustics_2D
    from . import acoustics_mapped_2D
    from . import acoustics_2D_ptwise
    from . import advection_2D
    from . import burgers_2D
    from . import euler_mapgrid_2D
    from . import euler_5wave_2D
    from . import euler_4wave_2D
    from . import euler_hlle_2D
    from . import euler_hlle_with_walls_2D
    from . import kpp_2D
    from . import psystem_2D
    from . import shallow_hlle_2D
    from . import shallow_roe_with_efix_2D
    from . import shallow_bathymetry_fwave_2D
    from . import sw_aug_2D
    from . import shallow_sphere_2D
    from . import vc_acoustics_2D
    from . import vc_advection_2D
    from . import vc_elasticity_2D
    from . import vc_acoustics_3D
    from . import euler_3D
    from . import burgers_3D
    from . import vc_advection_3D
    from . import euler_mapgrid_3D
except ImportError as e:
    import traceback
    print("********************************************************************")
    print('Warning: Some Riemannn solvers were not able to be imported.')
    print(' Did you run "pip install" in your clawpack directory?')
    traceback.print_exc()
    print("********************************************************************")

import os
if os.path.exists('./layered_shallow_water_1D.so'):
    import layered_shallow_water_1D
    from . import layered_shallow_water_1D_constants



#!/usr/bin/env python
# encoding: utf-8

r"""
One-dimensional advection
=========================

Solve the linear advection equation on a nonuniform grid:

.. math::
    q_t + u q_x = 0.

Here q is the density of some conserved quantity and u is the velocity.
Here we have a nonuniform grid, given by the transformation x**2 on grid [-0.5,0.5] to [-0.25,0.25].
The initial condition is a Gaussian centered at 0 and the boundary conditions are periodic.
The final solution is identical to the initial data because the wave has
crossed the domain exactly once, which takes computational time 0.5, because the speed is 1 and grid length 0.5.
"""

import numpy as np
from clawpack import riemann

def mapc2p_nonunif(xc):
    """This function takes the interval [-xL,xR] and squares the computational coordinate
       while keeping the negative coordinates to be squared and yet retain their negativity
       to the physical coordinate [-xL**2, xR**2]
    """
    neg = -1*(xc < 0) + (xc > 0)
    xp = xc**2
    xp = neg*xp
    return xp


def setup(nx=100, kernel_language='Fortran', use_petsc=False, solver_type='classic', weno_order=5,
          time_integrator='SSP104', outdir='./_output'):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    from clawpack import pyclaw
    import clawpack.pyclaw.geometry

    if kernel_language == 'Fortran':
        riemann_solver = riemann.advection_1D
    elif kernel_language == 'Python':
        riemann_solver = riemann.advection_1D_py.advection_1D

    # sharpclaw does not accommodate nonuniform grids
    if solver_type=='classic':
        solver = pyclaw.ClawSolver1D(riemann_solver)
    else: raise Exception('Unrecognized value of solver_type.')

    solver.kernel_language = kernel_language
    solver.order = 1 
    solver.limiters = pyclaw.tvd.minmod
    solver.num_eqn=1
    solver.num_waves=1
    solver.bc_lower[0] = pyclaw.BC.periodic
    solver.bc_upper[0] = pyclaw.BC.periodic
    solver.aux_bc_lower[0] = pyclaw.BC.periodic
    solver.aux_bc_upper[0] = pyclaw.BC.periodic

    x = pyclaw.Dimension(-0.5,0.5,nx,name='x')
    domain = pyclaw.Domain(x)
    state = pyclaw.State(domain,1,num_aux=1)
    state.problem_data['u'] = 1.  # Advection velocity
    state.index_capa = 0

    xc = state.grid.x.centers
    grid1d = state.grid

    # mapping to nonunif grid
    grid1d.mapc2p = mapc2p_nonunif
    state.aux = np.zeros((1,nx)) # capacity array dx_p/dx_c
    state.aux[0,:] = np.diff(grid1d.p_nodes)/np.diff(state.grid.x.nodes)

    # Initial data
    beta = 100; gamma = 0; x0 = 0.0
    state.q[0,:] = np.exp(-beta * (grid1d.p_centers-x0)**2) * np.cos(gamma * (grid1d.p_centers - x0))

    claw = pyclaw.Controller()
    claw.keep_copy = True
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver

    claw.tfinal = 0.5 # one cycle
    claw.outdir = outdir
    claw.num_output_times = 10
    claw.nstepout = 1
    if outdir is None:
        claw.output_format = None

    claw.setplot = setplot

    return claw

def setplot(plotdata):
    """
    Plot solution using VisClaw.
    """
    plotdata.clearfigures()  # clear any old figures,axes,items data
    plotdata.mapc2p = mapc2p_nonunif
    plotfigure = plotdata.new_plotfigure(name='q', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-0.25,0.25]
    plotaxes.ylimits = [-.2,1.0]
    plotaxes.title = 'q'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = '-o'
    plotitem.color = 'b'
    plotitem.kwargs = {'linewidth':2,'markersize':5}
    return plotdata

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)




"""
Module to create topo and qinit data files for this example.
"""

from __future__ import absolute_import
from clawpack.geoclaw.topotools import Topography
from numpy import *

def maketopo():
    """
    Output topography file for the entire domain
    """
    nxpoints = 201
    nypoints = 201
    xlower = -1000.e0
    xupper = 1000.e0
    yupper = 1000.e0
    ylower = -1000.e0
    outfile= "bowl.topotype2"     

    topography = Topography(topo_func=topo)
    topography.x = linspace(xlower,xupper,nxpoints)
    topography.y = linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=2, Z_format="%22.15e")

def makeqinit():
    """
    Create qinit data file
    """
    nxpoints = 101
    nypoints = 101
    xlower = -100.e0
    xupper = 100.e0
    yupper = 100.e0
    ylower = -100.e0
    outfile= "hump.xyz"     

    topography = Topography(topo_func=qinit)
    topography.x = linspace(xlower,xupper,nxpoints)
    topography.y = linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=1)

def topo(x,y):
    """
    Parabolic bowl
    """
    # value of z at origin:  Try zmin = 80 for shoreline or 250 for no shore
    zmin = 80.
    z = 1.e-2*(x**2 + y**2) - zmin

    # r = x**2 + y**2
    # z = where(r<80**2, -zmin, zmin)
    return z


def qinit(x,y):
    """
    Gaussian hump:
    """
    from numpy import where
    ze = -((x+0e0)**2 + (y+0e0)**2)/100
    z = where(ze>-10, 4.0*exp(ze), 0.)
    return z

if __name__=='__main__':
    maketopo()
    makeqinit()



# encoding: utf-8
r"""
Plotting routines for storm surge simulations with GeoClaw

:Authors:
    Kyle Mandli (2012-10-23) Initial version
"""
# ==============================================================================
#  Distributed under the terms of the Berkeley Software Distribution (BSD)
#  license
#                     http://www.opensource.org/licenses/
# ==============================================================================

from __future__ import absolute_import
from __future__ import print_function

import warnings

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.lines as mlines
import pandas

import clawpack.visclaw.colormaps as colormaps
import clawpack.visclaw.gaugetools as gaugetools
import clawpack.visclaw.geoplot as geoplot
import clawpack.geoclaw.data as geodata
# import clawpack.geoclaw.surge.storm

# TODO:  Assign these based on data files
bathy_index = 0
friction_field = 3
wind_field = 4
pressure_field = 6

surface_cmap = plt.get_cmap("bwr")
speed_cmap = plt.get_cmap('PuBu')
friction_cmap = plt.get_cmap('YlOrRd')
velocity_cmap = plt.get_cmap('PiYG')
vorticity_cmap = plt.get_cmap('PRGn')
wind_cmap = plt.get_cmap('PuBu')
pressure_cmap = plt.get_cmap('PuBu')
land_cmap = geoplot.land_colors

surge_data = geodata.SurgeData()

# ==============================
#  Track Plotting Functionality
# ==============================
class track_data(object):
    """Read in storm track data from run output"""

    def __init__(self, path=None):
        if path is None:
            path = "fort.track"

        try:
            self._path = path
            self._data = np.loadtxt(self._path)
        except:
            self._data = None

    def get_track(self, frame):
        """Return storm location for frame requested"""

        # If data was not load successfully return None
        if self._data is None or len(self._data.shape) < 2:
            return None, None, None

        # If it appears that our data is not long enough, try reloading file
        if self._data.shape[0] < frame + 1:
            self._data = np.loadtxt(self._path)

            # Check to make sure that this fixed the problem
            if self._data.shape[0] < frame + 1:
                warnings.warn(f" *** WARNING *** Could not find track data",
                               " for frame {frame}.")
                return None, None, None

        return self._data[frame, 1:]


# ==========================================================================
# Gauge functions
# ==========================================================================
def gauge_locations(current_data, gaugenos='all'):
    gaugetools.plot_gauge_locations(current_data.plotdata,
                                    gaugenos=gaugenos, format_string='kx',
                                    add_labels=True, xoffset=0.02,
                                    yoffset=0.02)


def gauge_dry_regions(cd, dry_tolerance=1e-16):
    """Masked array of zeros where gauge is dry."""
    return np.ma.masked_where(np.abs(cd.gaugesoln.q[0, :]) > dry_tolerance,
                              np.zeros(cd.gaugesoln.q[0, :].shape))


def gauge_surface(cd, dry_tolerance=1e-16):
    """Sea surface at gauge masked when dry."""
    return np.ma.masked_where(np.abs(cd.gaugesoln.q[0, :]) < dry_tolerance,
                              cd.gaugesoln.q[3, :])


def plot_landfall_gauge(gauge, axes, landfall=0.0, style='b', kwargs={}):
    """Plot gauge data on the axes provided

    This will transform the plot so that it is relative to the landfall value
    provided.

    This can be done using `plotaxes.time_scale` instead so this function will
    be deprecated and removed in a future release.
    """
    axes = plt.gca()

    # Add GeoClaw gauge data
    t = sec2days(gauge.t - landfall)
    axes.plot(t, gauge.q[3, :], style, **kwargs)


# ========================================================================
#  Surge related helper functions
# ========================================================================
def days_figure_title(current_data, land_fall=0.0):
    t = (current_data.t - land_fall) / (60**2 * 24)
    days = int(t)
    hours = (t - int(t)) * 24.0

    title = current_data.plotaxes.title
    plt.title('%s at day %3i, hour %2.1f' % (title, days, hours))


def surge_afteraxes(current_data, track, land_fall=0.0, plot_direction=False,
                    style='ro', kwargs={}):
    """Default surge plotting after axes function

    Includes changing the title to something relative to landfall and plotting
    the location of the storm eye according to the track object.
    """
    track_data = track.get_track(current_data.frameno)

    if track_data[0] is not None and track_data[1] is not None:
        axes = plt.gca()
        axes.plot(track_data[0], track_data[1], style, **kwargs)
        if plot_direction:
            axes.quiver(track_data[0], track_data[1],
                        np.cos(track_data[2]), np.sin(track_data[2]))
    days_figure_title(current_data, land_fall)


def friction(cd):
    return cd.aux[friction_field, :, :]


def wind_x(cd):
    # print(cd.aux[wind_field, :, :])
    return cd.aux[wind_field, :, :]


def wind_y(cd):
    # print(cd.aux[wind_field+1, :, :])
    return cd.aux[wind_field+1, :, :]


def wind_speed(cd):
    return np.sqrt(wind_x(cd)**2 + wind_y(cd)**2)


def pressure(cd):
    # The division by 100.0 is to convert from Pa to millibars
    return cd.aux[pressure_field, :, :] / 100.0


def storm_radius(cd, track):
    """Distance from center of storm"""
    track_data = track.get_track(cd.frameno)

    if track_data[0] is not None and track_data[1] is not None:
        return np.sqrt((cd.x - track_data[0])**2 + (cd.y - track_data[1])**2)
    else:
        return None


# ========================================================================
#  Water helper functions
# ========================================================================
def b(cd):
    return cd.aux[bathy_index, :, :]


def extract_eta(h, eta, DRY_TOL=1e-3):
    index = np.nonzero((np.abs(h) < DRY_TOL) + (h == np.nan))
    eta[index[0], index[1]] = np.nan
    return eta


def extract_velocity(h, hu, DRY_TOL=1e-8):
    u = np.zeros(hu.shape)
    index = np.nonzero((np.abs(h) > DRY_TOL) * (h != np.nan))
    u[index[0], index[1]] = hu[index[0], index[1]] / h[index[0], index[1]]
    return u


def eta(cd):
    return extract_eta(cd.q[0, :, :], cd.q[3, :, :])


def water_u(cd):
    return extract_velocity(cd.q[0, :, :], cd.q[1, :, :])


def water_v(cd):
    return extract_velocity(cd.q[0, :, :], cd.q[2, :, :])


def water_speed(current_data):
    u = water_u(current_data)
    v = water_v(current_data)

    return np.sqrt(u**2+v**2)


# ========================================================================
#  Plot items
# ========================================================================
def add_surface_elevation(plotaxes, plot_type='pcolor', bounds=None,
                          contours=None, shrink=1.0):
    """Add plotitem representing the sea surface."""

    if plot_type == 'pcolor' or plot_type == 'imshow':
        plotitem = plotaxes.new_plotitem(name='surface', plot_type='2d_pcolor')
        plotitem.plot_var = geoplot.surface_or_depth

        if bounds is not None:
            if bounds[0] == 0.0:
                plotitem.pcolor_cmap = plt.get_cmap('OrRd')
            else:
                plotitem.pcolor_cmap = surface_cmap
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = True
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Surface Height (m)"
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 0, 0, 0, 0]

    elif plot_type == 'contour':
        plotitem = plotaxes.new_plotitem(name='surface',
                                         plot_type='2d_contour')
        if bounds is None:
            plotitem.contour_levels = [-2.5, -1.5, -0.5, 0.5, 1.5, 2.5]

        plotitem.plot_var = geoplot.surface_or_depth
        plotitem.amr_contour_show = [1] * 10
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 0, 0, 0]
        plotitem.amr_contour_colors = 'k'

    elif plot_type == 'contourf':
        plotitem = plotaxes.new_plotitem(name='surface',
                                         plot_type='2d_contourf')
        plotitem.plot_var = geoplot.surface_or_depth
        if bounds is not None:
            contours = numpy.linspace(bounds[0], bounds[1], 11)
            plotitem.contour_levels = contours
            plotitem.fill_cmin = bounds[0]
            plotitem.fill_cmax = bounds[1]
        elif contours is not None:
            plotitem.contour_levels = contours
            plotitem.fill_cmin = min(contours)
            plotitem.fill_cmax = max(contours)

        plotitem.add_colorbar = True
        plotitem.fill_cmap = geoplot.tsunami_colormap
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Surface Height (m)"
        plotitem.fill_cmap = plt.get_cmap('OrRd')
        if any((value < 0 for value in plotitem.contour_levels)):
            plotitem.fill_cmap = surface_cmap

        plotitem.amr_contour_show = [1] * 10
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 0, 0, 0]
        plotitem.amr_contour_colors = 'k'


def add_speed(plotaxes, plot_type='pcolor', bounds=None,  contours=None,
              shrink=1.0):
    """Add plotitem representing speed of the water."""
    if plot_type == 'pcolor' or plot_type == 'imshow':
        plotitem = plotaxes.new_plotitem(name='speed', plot_type='2d_pcolor')
        plotitem.plot_var = water_speed
        # plotitem.plot_var = 1
        plotitem.pcolor_cmap = speed_cmap
        if bounds is not None:
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = True
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Current (m/s)"
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 0, 0, 0, 0]

    elif plot_type == 'contour':
        plotitem = plotaxes.new_plotitem(name='speed', plot_type='2d_contour')
        if bounds is None:
            plotitem.contour_levels = [0.5, 1.5, 3, 4.5, 6.0]
        plotitem.kwargs = {'linewidths': 1}

        plotitem.plot_var = water_speed
        plotitem.amr_contour_show = [1] * 10
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]
        plotitem.amr_contour_colors = 'k'

    elif plot_type == 'contourf':

        plotitem = plotaxes.new_plotitem(name='speed', plot_type='2d_contourf')

        plotitem.add_colorbar = True
        plotitem.colorbar_label = "Current (m/s)"
        plotitem.colorbar_shrink = shrink
        plotitem.fill_cmap = plt.get_cmap('PuBu')
        if bounds is not None:
            plotitem.contour_levels = numpy.linspace(bounds[0], bounds[1], 11)
            plotitem.fill_cmin = bounds[0]
            plotitem.fill_cmap = bounds[1]
        elif contours is not None:
            plotitem.contour_levels = contours
            plotitem.fill_cmin = min(contours)
            plotitem.fill_cmax = max(contours)

        # Modify the 'extends' plot attribute as we don't want this to extend
        # below 0
        plotitem.kwargs['extend'] = 'max'

        plotitem.plot_var = water_speed
        plotitem.amr_contour_show = [1] * 10
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]
        plotitem.amr_contour_colors = 'k'


def add_friction(plotaxes, bounds=None, plot_type='pcolor', shrink=1.0):
    """Add plotitem for the friction field"""

    if plot_type == 'pcolor' or plot_type == 'imshow':
        plotitem = plotaxes.new_plotitem(name='friction',
                                         plot_type='2d_pcolor')
        plotitem.plot_var = friction
        plotitem.pcolor_cmap = friction_cmap
        plotitem.colorbar_shrink = shrink
        if bounds is not None:
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = True
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Manning's-$n$ Coefficient"
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [0] * 10


def add_wind(plotaxes, bounds=None, plot_type='pcolor', shrink=1.0):
    """Add plotitem for the wind speed."""

    if plot_type == 'pcolor' or plot_type == 'imshow':
        plotitem = plotaxes.new_plotitem(name='wind', plot_type='2d_pcolor')
        plotitem.plot_var = wind_speed
        plotitem.pcolor_cmap = wind_cmap
        if bounds is not None:
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = True
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Wind Speed (m/s)"
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]

    elif plot_type == 'contour':
        plotitem = plotaxes.new_plotitem(name='wind', plot_type='2d_contour')
        plotitem.plot_var = wind_speed
        plotitem.contour_nlevels = len(surge_data.wind_refine)
        plotitem.countour_min = surge_data.wind_refine[0]
        plotitem.patchedges_show = 1


def add_pressure(plotaxes, bounds=None, plot_type='pcolor', shrink=1.0):
    """Add plotitem for the pressure field."""

    if plot_type == 'pcolor' or plot_type == 'imshow':
        plotitem = plotaxes.new_plotitem(name="pressure",
                                         plot_type='2d_pcolor')
        plotitem.plot_var = pressure
        plotitem.colorbar_shrink = shrink
        plotitem.pcolor_cmap = pressure_cmap
        if bounds is not None:
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = True
        plotitem.colorbar_shrink = shrink
        plotitem.colorbar_label = "Pressure (mbar)"
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]
    elif plot_type == 'contour':
        pass


def add_land(plotaxes, plot_type='pcolor', bounds=None):
    """Add plotitem for land"""

    if plot_type == 'pcolor':
        plotitem = plotaxes.new_plotitem(name='land', plot_type='2d_pcolor')
        plotitem.show = True
        plotitem.plot_var = geoplot.land
        plotitem.pcolor_cmap = land_cmap
        if bounds is not None:
            plotitem.pcolor_cmin = bounds[0]
            plotitem.pcolor_cmax = bounds[1]
        plotitem.add_colorbar = False
        plotitem.amr_celledges_show = [0] * 10
        plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]

    elif plot_type == 'contour':
        plotitem = plotaxes.new_plotitem(name="land", plot_type='2d_contour')
        plotitem.plot_var = geoplot.land
        plotitem.contour_nlevels = 40
        plotitem.contour_min = bounds[0]
        plotitem.contour_max = bounds[1]
        plotitem.amr_contour_colors = ['g']  # color on each level
        plotitem.amr_patch_bgcolor = ['#ffeeee', '#eeeeff', '#eeffee']
        plotitem.celledges_show = 0
        plotitem.patchedges_show = 0


def add_bathy_contours(plotaxes, contour_levels=None, color='k'):
    """Add plotitem to plot contours of the topography"""

    plotitem = plotaxes.new_plotitem(name='bathy', plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    if contour_levels is None:
        contour_levels = [0.0]
    plotitem.contour_levels = contour_levels
    plotitem.amr_contour_colors = [color]
    plotitem.kwargs = {'linestyles': 'solid', 'linewidths': 2}
    plotitem.amr_contour_show = [1] * 10
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0


def add_storm_radii(plotaxes, track, radii=[100e3], color='r'):
    """Add radii to plots based on storm position"""
    plotitem = plotaxes.new_plotitem(name="storm radius", 
                                     plot_type="2d_contour")
    plotitem.plot_var = lambda cd: storm_radius(cd, track)
    plotitem.contour_levels = radii
    plotitem.contour_colors = color


# ===== Storm related plotting =======
def sec2days(seconds):
    """Converst seconds to days."""
    return seconds / (60.0**2 * 24.0)


def plot_track(t, x, y, wind_radius, wind_speed, Pc, name=None):
    r"""Plot hurricane track given a storm.data file"""

    if name is None:
        name = ""
    else:
        name = " - %s" % name

    colors = ['r', 'b']
    divide = (np.max(Pc) + np.min(Pc)) / 2.0

    fig = plt.figure(1)
    axes = fig.add_subplot(111)
    indices = Pc < divide
    axes.scatter(x[indices], y[indices], color='r', marker='o')
    indices = Pc >= divide
    axes.scatter(x[indices], y[indices], color='b', marker='o')
    axes.set_title("Track%s" % name)

    fig = plt.figure(2, figsize=(24, 6))
    axes = fig.add_subplot(131)
    axes.plot(sec2days(t), wind_speed)
    axes.set_title("Maximum Wind Speed%s" % name)

    axes = fig.add_subplot(132)
    axes.plot(sec2days(t), wind_radius)
    axes.set_title("Maximum Wind Radius%s" % name)

    axes = fig.add_subplot(133)
    axes.plot(sec2days(t), Pc)
    axes.plot(sec2days(t), np.ones(t.shape) * divide, 'k--')
    axes.set_title("Central Pressure%s" % name)


"""Plot the track and optionally the intensity of the storm

Easily plot the track and intensity of a storm using a mapping package.

:Input:
    - *axes* (matplotlib.pyplot.axes) Axes to plot into.  Default is *None*
    - *intensity* (bool) Plot the intensity of storm along the track.
    Defaults to *False*.
    - *track_color* (str) String or specification of plotting color to use
    for the track if *intensity* is not being plotted.
    - *category_color* (dict) Dictionary containing mapping between
    category numerical value and colors.  Defaults to [-1, 5] -> ['gray',
    'black', 'violet', 'blue', 'yellow', 'orange', 'red']
    - *categorization* (str) Type of categorization, always use *"NHC"*
- *legend_loc* (str) Location of legend. Available options are 'best' (0), 'upper right' (1),
    'upper left'(2), 'lower left'(3), 'lower right'(4), 'right'(5), 'center left'(6), 'center right'(7), 
    'lower center'(8), 'upper center'(9), 'center'(10). Default is 'best'.

:Output:
    - (matplotlib.pyplot.axes) Axes object that was plotted into.
"""
# ========================================================================
#  Returns axes
#  Storm with category plotting function
# ========================================================================


def add_track(Storm, axes, plot_package=None, category_color=None, legend_loc='best',
              intensity=False, categorization="NHC", limits=None, track_color='red'):

    if category_color is None:
        category_color = {5: 'red',
                          4: 'orange',
                          3: 'yellow',
                          2: 'blue',  # edit color
                          1: 'violet',
                          0: 'black',
                          -1: 'gray'}
    category = Storm.category(categorization=categorization)

    # make it if intensity = true

    # basic plotting
    longitude = Storm.eye_location[:, 0]
    latitude = Storm.eye_location[:, 1]
    for i in range(len(longitude)):
        if intensity:
            color = category_color[category[i]]
        else:
            color = track_color
        axes.plot(longitude[i:i + 2], latitude[i:i + 2], color=color)

    axes.set_xlabel("Longitude")
    axes.set_ylabel("Latitude")

    categories_legend = []

    if intensity and categorization == "NHC":
        categories_legend = []
        # plotitem = plotaxes.new_plotitem(name='category', plot_type='1d_plot')

        if (-1 in category):
            negativeone = mlines.Line2D(
                [], [], color=category_color[-1], marker='s', ls='', label="Tropical Depression")
            categories_legend.append(negativeone)

        if (0 in category):
            zero = mlines.Line2D(
                [], [], color=category_color[0], marker='s', ls='', label="Tropical Storn")
            categories_legend.append(zero)

        if (1 in category):
            one = mlines.Line2D([], [], color=category_color[1],
                                marker='s', ls='', label="Category 1")
            categories_legend.append(one)

        if (2 in category):
            two = mlines.Line2D([], [], color=category_color[2],
                                marker='s', ls='', label="Category 2")
            categories_legend.append(two)

        if (3 in category):
            three = mlines.Line2D(
                [], [], color=category_color[3], marker='s', ls='', label="Category 3")
            categories_legend.append(three)

        if (4 in category):
            four = mlines.Line2D(
                [], [], color=category_color[4], marker='s', ls='', label="Category 4")
            categories_legend.append(four)

        if (5 in category):
            five = mlines.Line2D(
                [], [], color=category_color[5], marker='s', ls='', label="Category 5")
            categories_legend.append(five)

        plt.legend(handles=categories_legend, loc=legend_loc)

    # if bounds is not None:
    #     plotitem.pcolor_cmin = bounds[0]
    #     plotitem.pcolor_cmax = bounds[1]

    return axes

    # if plot_type == 'pcolor' or plot_type == 'imshow':
    #     plotitem = plotaxes.new_plotitem(name='wind', plot_type='2d_pcolor')
    #     plotitem.plot_var = wind_speed
    #     plotitem.pcolor_cmap = wind_cmap
    #     if bounds is not None:
    #         plotitem.pcolor_cmin = bounds[0]
    #         plotitem.pcolor_cmax = bounds[1]
    #     plotitem.add_colorbar = True
    #     plotitem.colorbar_shrink = shrink
    #     plotitem.colorbar_label = "Wind Speed (m/s)"
    #     plotitem.amr_celledges_show = [0] * 10
    #     plotitem.amr_patchedges_show = [1, 1, 1, 1, 1, 0, 0]

    # elif plot_type == 'contour':
    #     plotitem = plotaxes.new_plotitem(name='wind', plot_type='2d_contour')
    #     plotitem.plot_var = wind_speed
    #     plotitem.contour_nlevels = len(surge_data.wind_refine)
    #     plotitem.countour_min = surge_data.wind_refine[0]
    #     plotitem.patchedges_show = 1



"""
Functions to take a rectangular patch of data, or an AMRClaw framesoln
containing many patches, and extract values on a new single grid.
Can be used to generate a uniform grid from an AMR solution, or to extract
a 1d transect (or values at any given set of points) from a 2d patch or
framesoln.


:Functions:
  - grid_eval_2d: take single patch of 2d data and evaluate on new grid
  - grid_output_2d: take 2d (AMR) solution and evaluate on new grid
 
:Todo:
  - extend to 1d and 3d inputs.

"""

from __future__ import print_function
import numpy as np

def grid_eval_2d(X, Y, Q, xout, yout, method='nearest', return_ma=True):
    """
    Utility function that takes a single patch of data in 2d and
    returns values on 1d or 2d grid specified by xout, yout.

    Input:
        arrays X,Y defining a grid patch and data Q on this patch,
            Q can be a 2d array of the same shape as X,Y or a
            3d array with Q[m,:,:] corresponding to the m'th value at each point
        xout, yout defining the points for output (1d or 2d arrays)
        method: by default 'nearest', which samples piecewise constant 
            values in each cell.  Can also set to 'linear'
        return_ma (bool) determines if output is a masked array
    Returns:
        qout
        
    ndim(Q) is either 2 or 3.  If 3, then Q[m,i,j] is  m'th variable at i,j
    if ndim(xout)==ndim(yout)==1 then an arbitrary set of points can be
        specified (e.g. along a transect, or curve, or scattered).
    if ndim(xout)==ndim(yout)==2 then Q is interpolated to this grid of points.
    
    if return_ma==True then the result is masked at points outside
        the limits of X,Y.   Otherwise result is NaN at these points.
    Uses zero-order interpolation, i.e.
        Sets value qout[i,j] to value in the finite volume grid cell
        of X,Y that contains (xout[i,j],yout[i,j]).
    Future: allow bilinear interpolation instead but this requires
        ghost cell values around Q grid.  (These are present in binary
        output fort.b files but normally thrown away.)
    """

    from scipy.interpolate import RegularGridInterpolator
    from numpy import ma  # for masked arrays
    
    Qdim = Q.ndim
    if Qdim == 2:
        # change to 3d array of shape (1, Q.shape[0], Q.shape[1]):
        Q = np.array([Q])
        
    nvars = Q.shape[0]  # number of arrays to interpolate

    ndim_out = len(xout.shape)
    xout1 = np.ravel(xout)
    yout1 = np.ravel(yout)
    x1 = X[:,0]
    y1 = Y[0,:]
    dx = x1[1] - x1[0]
    dy = y1[1] - y1[0]
    if dx<=0 or dy<=0:
        raise ValueError('X[:,0],Y[0,:] must be increasing. ' \
                + 'Need to transpose arrays?')

    # augment Q with border of values on all 4 sides:
    x1 = np.hstack((x1[0]-0.501*dx, x1, x1[-1]+0.501*dx))
    y1 = np.hstack((y1[0]-0.501*dy, y1, y1[-1]+0.501*dy))
    Q1 = np.empty((nvars,len(x1),len(y1)))
    Q1[:,1:-1, 1:-1] = Q   # center portion
    Q1[:,1:-1,0] = Q[:,:,0]
    Q1[:,1:-1,-1] = Q[:,:,-1]
    Q1[:,0,1:-1] = Q[:,0,:]
    Q1[:,-1,1:-1] = Q[:,-1,:]
    # corners:
    Q1[:,0,0] = Q[:,0,0]
    Q1[:,0,-1] = Q[:,0,-1]
    Q1[:,-1,-1] = Q[:,-1,-1]
    Q1[:,-1,0] = Q[:,-1,0]

    qout = np.empty([nvars]+list(xout.shape))
    for k in range(nvars):
        evalfunc = RegularGridInterpolator((x1,y1), Q1[k,:,:], method=method,
                bounds_error=False, fill_value=np.nan)
        xyout = np.vstack((xout1,yout1)).T
        qout_k = evalfunc(xyout)
        if ndim_out == 2:
            qout_k = np.reshape(qout_k, xout.shape)   
            qout[k,:,:] = qout_k
        else:
            qout[k,:] = qout_k

    if Qdim==2 and ndim_out==2:
        qout = qout[0,:,:]  # revert back to 2d array
    if Qdim==2 and ndim_out==1:
        qout = qout[0,:]  # revert back to 1d array

    if return_ma:
        # convert from an array with nan's to a masked array:
        qout = ma.masked_where(qout != qout, qout)

    #print('type is %s' % type(qout))

    return qout
    

def grid_output_2d(framesoln, out_var, xout, yout, levels='all', 
                   method='nearest', return_ma=True):

    """
    :Input:
        framesoln:  One frame of Clawpack solution (perhaps with AMR),
                 An object of type pyclaw.Solution.solution.
        out_var: function that maps q to desired quantities Q[m,i,j] or
                 Q[i,j] if only one.  
                 If type(out_var) == int, then Q[i,j] = q[out_var,i,j]
        xout, yout: arrays of output points (1d or 2d arrays)
        levels: list of levels to use, or 'all'
        method: by default 'nearest', which samples piecewise constant 
            values in each cell.  Can also set to 'linear'
        return_ma: True to return as masked_array, False to return with
                NaN in locations that framesoln doesn't cover.
    :Output:
        qout: Solution obtained on xout,yout grid

    Loop over all patches in framesoln and apply grid_eval function.
    Use non-NaN values that this returns to update qout array over the
        region covered by this patch

    :Example:

    Note that one frame of a Clawpack simulation can be loaded via, e.g.:

        from clawpack.pyclaw.solution import Solution
        framesoln = Solution(frameno=1, path='_output', 
                             file_format='ascii')

    Then define `xout, yout, out_var` and call this function.

    """
        
    from numpy import ma  # for masked arrays
    if levels == 'all':
        levels = range(1,100)  # more levels than will ever use

    qout = np.empty(xout.shape)
    qout[:] = np.nan
    xmin = xout.min()
    xmax = xout.max()
    ymin = yout.min()
    ymax = yout.max()
    for stateno,state in enumerate(framesoln.states):
        state = framesoln.states[stateno]
        patch = state.patch
        #print('level = ',patch.level)

        if patch.level not in levels:
            # skip this patch
            continue

        if (xmin > state.grid.x.upper) or (xmax < state.grid.x.lower) \
                or (ymin > state.grid.y.upper) or (ymax < state.grid.y.lower):
            # no overlap
            continue
            
        #print('overlap at level %i' % patch.level)
        Xc,Yc = state.grid.c_centers
        xc = Xc[:,0]
        yc = Yc[0,:]

        if type(out_var) == int:
            Q = state.q[out_var, :, :]
        else:
            Q = out_var(state.q)

        qout1 = grid_eval_2d(Xc, Yc, Q, xout, yout, method=method, 
                             return_ma=False)
        qout = np.where(np.isnan(qout1), qout, qout1)
        if return_ma:
            # convert from an array with nan's to a masked array:
            qout = ma.masked_where(qout != qout, qout)
    return qout




#!/usr/bin/env python
# encoding: utf-8
r"""
Module containing forestclaw.geometry.
All we do here is override the pyclaw.Patch class to add a block_number and mpi_rank.
"""

from __future__ import absolute_import

from clawpack import pyclaw
from clawpack.pyclaw import geometry as pyclaw_geometry
from clawpack.pyclaw.geometry import Dimension
from clawpack.pyclaw.geometry import Domain


class Patch(pyclaw_geometry.Patch):
    """Patch class with specific ForestClaw attributes.
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw_geometry.Patch)

    def __init__(self, dimensions):

        super(Patch, self).__init__(dimensions)

        self.block_number = 0
        r"""(int) - Block number of current patch, ``default = 0``"""
        self.mpi_rank = 0
        r"""(int) - MPI rank this patch belongs to, ``default = 0``"""




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

import numpy as np
import matplotlib.pyplot as plt

from clawpack.geoclaw import topotools

try:
    TG32412 = np.loadtxt('32412_notide.txt')
except:
    print("*** Could not load DART data file")


#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from clawpack.visclaw import colormaps, geoplot
    from numpy import linspace

    plotdata.clearfigures()  # clear any old figures,axes,items data


    # To plot gauge locations on pcolor or contour plot, use this as
    # an afteraxis function:

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos='all', format_string='ko', add_labels=True)
    
    # ========================================================================
    #  Water helper functions
    # ========================================================================
    def b(cd):
        return cd.q[3,:,:] - cd.q[0,:,:]
        
    def extract_eta(h,eta,DRY_TOL=10**-3):
        index = np.nonzero((np.abs(h) < DRY_TOL) + (h == np.nan))
        eta[index[0],index[1]] = np.nan
        return eta
    
    def extract_velocity(h,hu,DRY_TOL=10**-8):
        u = np.zeros(hu.shape)
        index = np.nonzero((np.abs(h) > DRY_TOL) * (h != np.nan))
        u[index[0],index[1]] = hu[index[0],index[1]] / h[index[0],index[1]]
        return u
    
    def eta(cd):
        return extract_eta(cd.q[0,:,:],cd.q[3,:,:])
        
    def water_u(cd):
        return extract_velocity(cd.q[0,:,:],cd.q[1,:,:])
        
    def water_v(cd):
        return extract_velocity(cd.q[0,:,:],cd.q[2,:,:])
        
    def water_speed(current_data):
        u = water_u(current_data)
        v = water_v(current_data)
            
        return np.sqrt(u**2+v**2)


    #-----------------------------------------
    # Figure for surface
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.title = 'Surface'
    plotaxes.scaled = True

    def fixup(current_data):
        import pylab
        addgauges(current_data)
        t = current_data.t
        t = t / 3600.  # hours
        pylab.title('Surface at %4.2f hours' % t, fontsize=20)
        pylab.xticks(fontsize=15)
        pylab.yticks(fontsize=15)
    plotaxes.afteraxes = fixup

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    #plotitem.plot_var = geoplot.surface
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.2
    plotitem.pcolor_cmax = 0.2
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [1,1,0]
    plotitem.patchedges_show = 1
    plotaxes.xlimits = [-120,-60]
    plotaxes.ylimits = [-60,0]

    # add contour lines of bathy if desired:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = linspace(-3000,-3000,1)
    plotitem.amr_contour_colors = ['y']  # color on each level
    plotitem.kwargs = {'linestyles':'solid','linewidths':2}
    plotitem.amr_contour_show = [1,0,0]  
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0

    #-----------------------------------------
    # Figure for velocities
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Speeds', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('speeds')
    plotaxes.title = 'Speeds'
    plotaxes.scaled = True

    def fixup(current_data):
        import pylab
        addgauges(current_data)
        t = current_data.t
        t = t / 3600.  # hours
        pylab.title('Speeds at %4.2f hours' % t, fontsize=20)
        pylab.xticks(fontsize=15)
        pylab.yticks(fontsize=15)
    plotaxes.afteraxes = fixup

    # Speed
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = water_speed
    plotitem.pcolor_cmap = plt.get_cmap('PuBu')
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 0.01
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [1,1,0]
    plotitem.patchedges_show = 1
    plotaxes.xlimits = [-120,-60]
    plotaxes.ylimits = [-60,0]


    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface at gauges', figno=300, \
                    type='each_gauge')
    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Surface'

    # Plot surface as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 3
    plotitem.plotstyle = 'b-'

    # Plot topo as green curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.show = False

    def gaugetopo(current_data):
        q = current_data.q
        h = q[0,:]
        eta = q[3,:]
        topo = eta - h
        return topo
        
    # plotitem.plot_var = gaugetopo
    # plotitem.plotstyle = 'g-'

    def add_zeroline(current_data):
        from pylab import plot, legend, xticks, floor, axis, xlabel
        t = current_data.t 
        gaugeno = current_data.gaugeno

        if gaugeno == 32412:
            try:
                plot(TG32412[:,0], TG32412[:,1], 'r')
                legend(['GeoClaw','Obs'],'lower right')
            except: pass
            axis((0,t.max(),-0.3,0.3))

        plot(t, 0*t, 'k')
        n = int(floor(t.max()/3600.) + 2)
        xticks([3600*i for i in range(n)], ['%i' % i for i in range(n)])
        xlabel('time (hours)')

    plotaxes.afteraxes = add_zeroline



    #-----------------------------------------
    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata




from __future__ import absolute_import
from __future__ import print_function
import RiemannSolver
from RiemannSolver import *
from numpy import empty
from pylab import plot,  figure, suptitle
from six.moves import range



print("Testing for different number of time steps ...")
noOfTestSamples = 10
max_mx = 101 
max_timeSteps = 101 

timesStepsValues = empty((noOfTestSamples ))
timeResultsVectorized =empty((noOfTestSamples ))
timeResultsPointwise =empty((noOfTestSamples ))
# error when max_mx is divisible by noOfTestSamples. size of arrays should be noOfTestSamples-1, use j to decide what to plot






rs = RiemannSolver(timeSteps = max_timeSteps/noOfTestSamples, mwaves = 2, mx=max_mx , meqn = 2, maux = 2)  


rs.q = random.random((rs.meqn,rs.mx))
rs.aux = random.random((rs.maux,rs.mx))

j = 0
for i in range(max_timeSteps/noOfTestSamples, max_timeSteps, max_timeSteps/noOfTestSamples):
	timesStepsValues[j] = i
	rs.timeSteps = i
	print()
	print("Iteration",j+1,"of", noOfTestSamples)
	print("Number of time steps is", i)
	
	waves1, s1  = rs.solveVectorized(timer = True)
	timeResultsVectorized[j] = rs.elapsedTime
	
	
	
	waves2, s2  = rs.solvePointwize(timer = True)
	timeResultsPointwise[j] = rs.elapsedTime
	
	
	j= j+1


figure(1)	
suptitle("Figure 1 shows the the execution time for different timeStep values\n for the vectorized solver in green and pointwize solver in blue")	

plot(timesStepsValues, timeResultsVectorized, "go-")

#figure(2)
#suptitle( "Figure 2 shows the the execution time for different timeStep values\n for the pointwize solver, mx = {0}".format(max_mx))	
plot(timesStepsValues, timeResultsPointwise, "bo-")
	
	




print("Testing for different sizes of mx ...")
noOfTestSamples = 10
max_mx = 101 
max_timeSteps = 101 

mxValues = empty((noOfTestSamples ))
timeResultsVectorized =empty((noOfTestSamples ))
timeResultsPointwise =empty((noOfTestSamples ))
# error when max_mx is divisible by noOfTestSamples. size of arrays should be noOfTestSamples-1, use j to decide what to plot






rs = RiemannSolver(timeSteps = max_timeSteps, mwaves = 2, mx=max_mx/noOfTestSamples , meqn = 2, maux = 2)  


rs.q = random.random((rs.meqn,rs.mx))
rs.aux = random.random((rs.maux,rs.mx))





 




j = 0
for i in range(max_mx/noOfTestSamples, max_mx,max_mx/noOfTestSamples):
	mxValues[j] = i
	rs.mx = i
	print()
	print("Iteration",j+1,"of", noOfTestSamples)
	print("mx is", i)
	
	rs.q = random.random((rs.meqn,rs.mx))
	rs.aux = random.random((rs.maux,rs.mx))
	
	waves1, s1  = rs.solveVectorized(timer = True)
	timeResultsVectorized[j] = rs.elapsedTime
	
	
	
	waves2, s2  = rs.solvePointwize(timer = True)
	timeResultsPointwise[j] = rs.elapsedTime
	
	
	j= j+1
	
figure(2)
suptitle("Figure 2 shows the the execution time for different mx values\n for the vectorized solver in green and pointwize solver in blue")	
plot(mxValues, timeResultsVectorized, "go-")

#figure(4)
#suptitle("Figure 4 shows the the execution time for different mx values\n for the pointwize solver, timeSteps = {0}".format(max_timeSteps))	
plot(mxValues, timeResultsPointwise, "bo-")







#!/usr/bin/env python

r"""Particles regression test for GeoClaw

To create new regression data use
    `python regression_tests.py True`
"""

from __future__ import absolute_import
import os
import sys
import unittest
import shutil

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools


class ParticlesTest(test.GeoClawRegressionTest):

    r"""Particles regression test for GeoClaw"""

    def setUp(self):

        super(ParticlesTest, self).setUp()
        start_dir = os.getcwd()

        # Make topography

        shutil.copy(os.path.join(self.test_path, "maketopo.py"),
                                 self.temp_path)
        os.chdir(self.temp_path)
        os.system('python maketopo.py')
        os.chdir(start_dir)


    def runTest(self, save=False, indices=(2, 3)):
        r"""Test particles example

        Note that this stub really only runs the code and performs no tests.

        """

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        import clawpack.pyclaw.gauges as gauges
        gauge = gauges.GaugeSolution(1, path=self.temp_path)
        print('+++ Gauge 1:\n', gauge.q)
        gauge = gauges.GaugeSolution(2, path=self.temp_path)
        print('+++ Gauge 2:\n', gauge.q)

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, indices=(1, 2))
        self.check_gauges(save=save, gauge_id=2, indices=(1, 2))
        self.success = True


if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = ParticlesTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()




from __future__ import absolute_import
import RiemannSolver
from RiemannSolver import *
from pylab import plot,  figure, suptitle

rs = RiemannSolver(timeSteps =1, mwaves = 2, mx = 800, meqn = 2, maux = 2)
    
rs.q = random.random((rs.meqn,rs.mx))
rs.aux = random.random((rs.maux,rs.mx))
waves1, s1  = rs.solveVectorized(timer = True)
waves2, s2  = rs.solvePointwize(timer = True)



figure(1)
waveIndex = 0
componentIndex = 0

suptitle("Figure 1: shows the curve of wave number {0}, for the component number {1}".format(waveIndex+1, componentIndex+2))	
plot(waves1[componentIndex,waveIndex,:], "g")



#!/usr/bin/env python
# encoding: utf-8
r"""
Traffic flow
============

Solve the Lighthill-Whitham-Richards (LWR) traffic flow model:

.. math::
    q_t + u (q(1-q))_x & = 0.

Here q is the density of cars, and u is a constant specifying the speed limit.
"""

from clawpack import riemann

def setup(use_petsc=0,outdir='./_output',solver_type='classic'):
    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver1D(riemann.traffic_1D)
    else:
        solver = pyclaw.ClawSolver1D(riemann.traffic_1D)

    solver.bc_lower[0] = pyclaw.BC.extrap
    solver.bc_upper[0] = pyclaw.BC.extrap

    x = pyclaw.Dimension(-1.0,1.0,500,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 1
    state = pyclaw.State(domain,num_eqn)

    grid = state.grid
    xc=grid.p_centers[0]

    state.q[0,:] = 0.75*(xc<0) + 0.1*(xc>0.) 

    state.problem_data['efix']=True
    state.problem_data['umax']=1.

    claw = pyclaw.Controller()
    claw.tfinal =2.0
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.outdir = outdir
    if outdir is None:
        claw.output_format = None
    claw.setplot = setplot
    claw.keep_copy = True

    return claw


def setplot(plotdata):
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for q[0]
    plotfigure = plotdata.new_plotfigure(name='q[0]', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.ylimits = [-0.1, 1.1]
    plotaxes.title = 'q[0]'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = 0
    plotitem.plotstyle = '-o'
    plotitem.color = 'b'
    
    return plotdata


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



"""
Useful tools for running Clawpack from an IPython notebook.
"""

from __future__ import absolute_import
from __future__ import print_function
from IPython.core.display import display
try:
    from IPython.display import FileLink
except:
    print("*** Ipython version does not support FileLink")


def make_driver(args, env, outfile, verbose):

    """
    Use subprocess to run a make command and capture the output.

    :Input:
    - *args*: arguments to the make command
    - *env*: environment to run in (if None, use *os.environ*)
      Useful if $CLAW must be set in notebook.
    - *outfile*: file name of file for capturing stdout and stderr
    - *vervose*: if True, print out command and display link to output file
    
    If the return code is non-zero, print a warning and link to output file
    regardless of value of *verbose*.

    """

    import subprocess
    import os, sys

    if env is None:
        env = os.environ

    if verbose:
        print("Executing shell command:   make %s" % args)
        sys.stdout.flush()

    ofile = open(outfile,'w')
    cmd_list = ['make'] + args.split()
    job = subprocess.Popen(cmd_list, stdout=ofile,stderr=ofile,env=env)
    return_code = job.wait()
    errors = (return_code != 0)
    if errors:
        print("*** Possible errors, return_code = %s" % return_code)
    
    if verbose or errors:
        local_file = FileLink(outfile)
        print("Done...  Check this file to see output:") 
        display(local_file)

def make_htmls(outfile=None, env=None, verbose=False, readme_link=True):
    """Perform 'make .htmls' and display link."""
    
    if outfile is None:
        outfile='htmls_output.txt'

    args = '.htmls'

    make_driver(args, env, outfile, verbose)

    if readme_link:
        print("See the README.html file for links to input files...")
        display(FileLink('README.html'))
    
def make_data(env=None, verbose=True):
    """Perform 'make data' and display link."""
    
    outfile='data_output.txt'
    args = 'data'
    make_driver(args, env, outfile, verbose)

def make_exe(new=False, env=None, verbose=True):
    """
    Perform 'make .exe' and display link.
    If *new = True*, do 'make new' instead to force recompilation of everything.
    """
    
    outfile='compile_output.txt'
    if new:
        args = 'new'
    else:
        args = '.exe'

    make_driver(args, env, outfile, verbose)

    
def make_output(label=None, env=None, verbose=True):
    """Perform 'make output' and display link."""
    
    if label is None: 
        label = ''
    else:
        if label[0] != '_':
            label = '_' + label
    outdir = '_output%s' % str(label)
    outfile = 'run_output%s.txt' % str(label)

    args = 'output OUTDIR=%s' % outdir
    make_driver(args, env, outfile, verbose)

    return outdir

    
def make_plots(label=None, env=None, verbose=True):
    """Perform 'make plots' and display links"""

    if label is None: 
        label = ''
    else:
        if label[0] != '_':
            label = '_' + label
    outdir = '_output%s' % str(label)
    plotdir = '_plots%s' % str(label)
    outfile = 'plot_output%s.txt' % str(label)

    args = 'plots OUTDIR=%s PLOTDIR=%s' % (outdir,plotdir)
    make_driver(args, env, outfile, verbose)

    if verbose:
        index_file = FileLink('%s/_PlotIndex.html' % plotdir)
        print("View plots created at this link:")
        display(index_file)

    return plotdir
    

def make_output_and_plots(label=None, env=None, verbose=True):

    outdir = make_output(label,env,verbose)
    plotdir = make_plots(label,env,verbose)
    return outdir,plotdir


def make_all(label=None, env=None, verbose=True):
    """Perform 'make all' and display links"""

    if label is None: 
        label = ''
    else:
        if label[0] != '_':
            label = '_' + label
    outdir = '_output%s' % str(label)
    plotdir = '_plots%s' % str(label)
    outfile = 'make_all_output%s.txt' % str(label)

    args = 'all OUTDIR=%s PLOTDIR=%s' % (outdir,plotdir)
    make_driver(args, env, outfile, verbose)

    if verbose:
        index_file = FileLink('%s/_PlotIndex.html' % plotdir)
        print("View plots created at this link:")
        display(index_file)

    return plotdir
    



from . import acoustics_1d
from clawpack.pyclaw.util import check_diff
import numpy as np

def error(**kwargs):
    """
    Compute difference between initial and final solutions.
    This should vanish due to the periodic boundary conditions
    """

    claw = acoustics_1d.setup(disable_output=True,**kwargs)
    claw.run()

    # tests are done across the entire domain of q normally
    q0 = claw.frames[0].state.get_q_global()
    qfinal = claw.frames[claw.num_output_times].state.get_q_global()

    q0 = q0.reshape([-1])
    qfinal = qfinal.reshape([-1])
    dx = claw.solution.domain.grid.delta[0]
    diff = dx*np.sum(np.abs(qfinal-q0))
    return diff

class TestRegression:
    def test_python_classic(self):
        assert abs(error(kernel_language='Python',solver_type='classic')-0.001981)<1e-5

    def test_fortran_classic(self):
        assert abs(error(kernel_language='Fortran',solver_type='classic')-0.001981)<1e-5

    def test_sharpclaw(self):
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw')-0.001540)<1e-5
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw',weno_order=11)-0.000521)<1e-5
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw',time_integrator='SSPLMMk3')-0.001545)<1e-5

class TestAccuracy:

    def test_python_classic(self):
        assert abs(error(num_cells=2000,kernel_language='Python',solver_type='classic'))<1e-5

    def test_fortran_classic(self):
        assert abs(error(num_cells=2000,kernel_language='Fortran',solver_type='classic'))<1e-5
        assert abs(error(num_cells=4000,kernel_language='Fortran',solver_type='classic'))<2e-6

    def test_sharpclaw(self):
        assert abs(error(num_cells=2000,kernel_language='Fortran',solver_type='sharpclaw'))<1e-8
        assert abs(error(num_cells=2000,kernel_language='Fortran',solver_type='sharpclaw',weno_order=11))<1e-8
        assert abs(error(num_cells=2000,kernel_language='Fortran',solver_type='sharpclaw',time_integrator='SSPLMMk3'))<2e-8



import os
import glob
import numpy as np
from clawpack.pyclaw import Solution
from clawpack.pyclaw.util import check_solutions_are_same

class IOTest():
    @property
    def solution(self):
        return Solution()

    @property
    def file_formats(self):
        return ['hdf5','ascii']

    @property
    def this_dir(self):
        return os.path.dirname(os.path.abspath(__file__))

    @property
    def test_data_dir(self):
        return os.path.join(self.this_dir, './test_data')

    def test_io_from_binary(self):
        # Read regression data
        regression_dir = os.path.join(self.test_data_dir,'./advection_2d_binary')
        self.read_write_and_compare(self.file_formats,regression_dir,'binary',0)

    def test_io_from_hdf5(self):
        regression_dir = os.path.join(self.test_data_dir,'./Sedov_regression_hdf')
        self.read_write_and_compare(self.file_formats,regression_dir,'hdf5',1)

    def test_io_from_hdf5_with_aux(self):
        regression_dir = os.path.join(self.test_data_dir,'./advection_2d_with_aux')
        self.read_write_and_compare(self.file_formats,regression_dir,'hdf5',0,aux=True)

    def read_write_and_compare(self, file_formats,regression_dir,regression_format,frame_num,aux=False):
        r"""Test IO file formats:
            - Reading in an HDF file
            - Writing  files in all formats (for now just ASCII, HDF5)
            - Reading those files back in
            - Checking that all the resulting Solution objects are identical
        """
        ref_sol = self.solution
        ref_sol.read(frame_num,path=regression_dir,file_format=regression_format,read_aux=aux)
        if aux:
            assert (ref_sol.state.aux is not None)

        # Write solution file in each format
        io_test_dir = os.path.join(self.this_dir,'./io_test')
        for fmt in file_formats:
            ref_sol.write(frame_num,path=io_test_dir,file_format=fmt,write_aux=aux)

        # Read solutions back in
        s = {}
        for fmt in file_formats:
            s[fmt] = self.solution
            s[fmt].read(frame_num,path=io_test_dir,file_format=fmt,write_aux=aux)

        # Compare solutions
        # Probably better to do this by defining __eq__ for each class
        for fmt, sol in s.items():
            check_solutions_are_same(sol,ref_sol)

    def test_io_to_vtk(self):
        # since the VTK only has a write and no read, I'm making sure that
        # the output matches sample output.

        # Input format (Binary)
        regression_dir = os.path.join(self.test_data_dir,'./advection_2d_binary')

        # read in from ascii.
        ref_sol = self.solution
        ref_sol.read(0,path=regression_dir,file_format="binary")

        # write out to vtk in io_test_dir
        io_test_dir = os.path.join(self.this_dir,'./io_test')
        ref_sol.write(0, file_format="vtk", path=io_test_dir)

        # Known correct data:
        comparison_dir = os.path.join(self.test_data_dir,'./advection_2d_vtk')

        # use glob to find all vthb and vti files.
        # compare  line by line.
        assert self.compare_vtk(comparison_dir, io_test_dir)==True

    def compare_vtk(self, dir1, dir2):
        # compare vtk files in two directories, return True if files are
        # equivalent.

        # assumption is that there are vthb files at top level and one level
        # down in folders are vthb files.

        # number of vthb and vti files is known. assert them.
        dir1_vthb_files = glob.glob(os.path.join(dir1, "*.vthb"))
        assert len(dir1_vthb_files) == 1

        dir1_vti_files = glob.glob(os.path.join(dir2, *["*", "*.vti"]))
        assert len(dir1_vti_files) == 17

        # for all files do line by line comparison.
        for dir1_file in dir1_vti_files + dir1_vthb_files:
            dir2_file = dir1_file.replace(dir1, dir2)

            # open files
            with open(dir1_file, 'r') as  f1:
                lines1 = f1.readlines()
            with open(dir2_file, 'r') as  f2:
                lines2 = f2.readlines()

            # compare each line.
            for l1, l2 in zip(lines1, lines2):
                assert l1.strip() == l2.strip()

            # delete lines for this file.
            del lines1, lines2

        # Only if all assertions succeed, return True.
        return True



from __future__ import absolute_import
from clawpack import pyclaw
from clawpack.pyclaw.solution import Solution

class Solution(Solution):
    """ Parallel Solution class.
    """
    __doc__ += pyclaw.util.add_parent_doc(pyclaw.Solution)

    def get_read_func(self, file_format):
        from clawpack.petclaw import fileio

        if file_format == 'petsc':
            try:
                import clawpack.petclaw.fileio
                return clawpack.petclaw.fileio.petsc.read
            except AttributeError as e:
                try:
                    from petsc4py import PETSc
                except ImportError:
                    raise ImportError("petsc4py is required for reading petsc format files, but petsc4py could not be imported.")
                raise e
        elif file_format == 'hdf5':
            return fileio.hdf5.read
        else:
            raise ValueError("File format %s not supported." % file_format)

    def get_write_func(self, file_format):
        from clawpack.petclaw import fileio
        if 'petsc' in file_format:
            return fileio.petsc.write
        elif 'hdf5' in file_format:
            return fileio.hdf5.write
        else:
            raise ValueError("File format %s not supported." % file_format)





from . import advection_1d_nonunif
import numpy as np
from clawpack.pyclaw.util import check_diff
import os

thisdir = os.path.dirname(__file__)
path_expected_sols = os.path.join(thisdir,'expected_sols.npy')
expected_sols_dict = np.load(path_expected_sols,allow_pickle=True).item()

def error(test_name,**kwargs):
    """
    Compute L1 norm of difference between test and expected
    solutions in the physical domain.
    """

    claw = advection_1d_nonunif.setup(outdir=None,**kwargs)
    claw.run()
    qtest = claw.frames[claw.num_output_times].state.get_q_global().reshape([-1])

    assert test_name in expected_sols_dict.keys(), f"Test name {test_name} not found in {path_expected_sols}"
    qexpected = expected_sols_dict[test_name]

    physical_nodes = claw.frames[0].state.grid.p_nodes
    dx=claw.solution.domain.grid.delta[0]
    comp_nodes = claw.frames[0].state.grid.c_nodes
    jac_mapc2p = np.diff(physical_nodes)/np.diff(comp_nodes)
    return np.sum(abs(dx*jac_mapc2p*(qtest-qexpected)))

class TestAdvectionNonUnif1D:
    def test_python_classic(self):
        assert error(test_name="python_classic",kernel_language='Python',solver_type='classic')<1e-6

    def test_fortran_classic(self):
        assert error(test_name="fortran_classic",kernel_language='Fortran',solver_type='classic')<1e-6



#!/usr/bin/env python
# encoding: utf-8
r"""
Defines common units used throughout GeoClaw along with functions for
converting between them.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import collections

from clawpack.geoclaw.data import LAT2METER

# Define baseline units so that we can convert to these and then back to
# requested.
standard_units = {'time': 's',
                  'spherical_length': 'lat-long',
                  'length': 'm',
                  'speed': 'm/s',
                  'radius': 'm',
                  'pressure': 'Pa',
                  'temperature': 'C'}

# Unit conversion definitions - handles conversions to standard units above
# The dictionary contains a list of values, the first is the conversion factor
# and the second a human readable version of the unit.
conversion_func = {}
units = {}

# Length
conversion_func['m'] = [lambda L: L,
                        lambda L: L]
conversion_func['cm'] = [lambda L: L * 1e-2,
                         lambda L: L / 1e-2]
conversion_func['km'] = [lambda L: L * 1e3,
                         lambda L: L / 1e3]
conversion_func['miles'] = [lambda L: L * 1.60934e3,
                            lambda L: L / 1.60934e3]
conversion_func['nmi'] = [lambda L: L * 1852.0,
                          lambda L: L / 1852.0]
conversion_func['lat-long'] = [lambda L: L * LAT2METER,
                               lambda L: L / LAT2METER]
units['length'] = collections.OrderedDict({'m': 'meters', 'cm': 'centimeters', 
                                           'km': 'kilometers', 'miles': 'miles',
                                           'nmi': 'nautical miles', 
                                           'lat-long': 'longitude-latitude'})

# Pressure - Rigidity
conversion_func['Pa'] = [lambda P: P, lambda P: P]
conversion_func['hPa'] = [lambda P: P * 1e2, 
                          lambda P: P / 1e2]
conversion_func['KPa'] = [lambda P: P * 1e3,
                          lambda P: P / 1e3]
conversion_func['MPa'] = [lambda P: P * 1e6,
                          lambda P: P / 1e6]
conversion_func['GPa'] = [lambda P: P * 1.e9,
                          lambda P: P / 1.e9]
conversion_func['mbar'] = [lambda P: P * 1e2,
                           lambda P: P / 1e2]
conversion_func['dyne/cm^2'] = [lambda P: P * 0.1,
                                lambda P: P / 0.1]
conversion_func['dyne/m^2'] = [lambda P: P * 1.e-5,
                               lambda P: P / 1.e-5]
units['pressure'] = {'Pa': 'pascals', 'hPa': 'hectopascals', 
                     'KPa': 'kilopascals', 'MPa': 'megapascals', 
                     'GPa': 'gigapascals', 'mbar': 'millibar', 
                     'dyne/cm^2': 'Dynes/cm^2', 'dyne/m^2': 'Dynes/m^2'}

# Speeds
conversion_func['km/h'] = [lambda v: v * 1e3 / (60**2), 
                           lambda v: v * 60**2 / 1e3]
conversion_func['mph'] = [lambda v: v * 0.44704, lambda v: v / 0.44704]
conversion_func['m/s'] = [lambda v: v, lambda v: v]
conversion_func['knots'] = [lambda v: v * 0.51444444, 
                            lambda v: v / 0.51444444]
units['speed'] = {'m/s': 'meters/second', 'knots': 'knots (nm / hour)', 
                  'km/h': 'kilometers/hour', 'mph': "miles / hour"}

# Moments
conversion_func['N-m'] = [lambda M: M, lambda M: M]
conversion_func['dyne-cm'] = [lambda M: M * 1.e-7,
                              lambda M: M / 1.e-7]
units['moment'] = {'N-m': "Newton-Meters", 'dyne-cm': "Dynes-Centimeter"}

# Temperature
conversion_func['C'] = [lambda temp: temp, lambda temp: temp]
conversion_func['F'] = [lambda temp: (temp - 32.0) * 5.0 / 9.0, 
                        lambda temp: temp * 9.0 / 5.0 + 32.0]
conversion_func['K'] = [lambda temp: temp + 273.15,
                        lambda temp: temp - 273.15]
units['temperature'] = {'C': "Celsius", 'F': "Fahrenheit", 'K': "Kelvin"}


def units_available():
    r"""
    Constructs a string suitable for reading detailing the units available.
    """
    output = ""
    for (measurement_type, measurement_units) in units.items():
        output = "\n".join((output, measurement_type.capitalize()))
        for (abbrv, full_name) in measurement_units.items():
            output = "\n".join((output, "  %s (%s)" % (abbrv, full_name)))

    return output


# Unit conversion function
def convert(value, old_units, new_units, verbose=False):
    r"""Convert *value* from *old_units* to *new_units*

    :Note:
    Currently this function only handles multiplicative conversions.  The
    reasoning behind not just returning this conversion factor as this function
    in the future will also support more complex unit conversions, e.g.
    converting between temperature scales.

    :Input:
     - *value* (ndarray or float) The value(s) to be converted.
     - *old_units* (string) Type of units that value comes in as.
     - *new_units* (string) Type of units that value should be converted to.
     - *verbose* (bool) Verbose output (default is False)

    :Output:
     - (ndarray or float) The converted value(s)
    """

    found_type = None
    for (measurement_type, measurement_units) in units.items():
        if old_units in measurement_units:
            found_type = measurement_type
            break

    if found_type is None:
        raise ValueError("Units %s not found in list of " % str(old_units),
                         "supported conversions." )

    if new_units not in units[found_type].keys():
        raise ValueError("Units %s not found in list of " % str(new_units),
                         "supported conversions of %s type." % found_type)

    if verbose:
        print("Convert %s %s to %s." % (value, old_units, new_units))

    return conversion_func[new_units][1](conversion_func[old_units][0](value))


if __name__ == '__main__':
    # Add commandline unit conversion capability
    if len(sys.argv) == 4:
        convert(float(sys.argv[1]), sys.argv[2], sys.argv[3])
    else:
        # Usage and available units
        print("Usage:  Convert value in units to new units.")
        print("  units <value> <old units> <new units>")
        print("where <old units> and <new units> are one of the available")
        print("units listed below.")
        print("")
        print("Available Units:")
        print("  First value is the abbreviation that should be used as an")
        print("  input unit while the second is the full name of the unit.")
        print(units_available())



"""
Conversion module for 2d classic, amrclaw, or geoclaw setrun.py file 
from 4.6 to 5.0 format.
"""
    
from __future__ import absolute_import
from __future__ import print_function
import os,sys
from permute import permute_file

try:
    clawutil = os.path.join(os.environ['CLAW'], 'clawutil/src/python/clawutil')
except:
    raise Exception("*** Environment variable CLAW must be set")


sys.path.insert(0,os.getcwd())
sys.path.append(os.getcwd() + '/pyclaw')

def convert_setrun(setrun_file='setrun.py', claw_pkg=None, ndim=None):

    setrun_text = open(setrun_file).readlines()
    if claw_pkg is None:
        for line in setrun_text:
            if 'claw_pkg' in line:
                if 'classic' in line:
                    claw_pkg = 'classic'
                elif 'amrclaw' in line:
                    claw_pkg = 'amrclaw'
                elif 'geoclaw' in line:
                    claw_pkg = 'geoclaw'

    if ndim is None:
        for line in setrun_text:
            if 'ndim' in line:
                if '1' in line: ndim = 1
                elif '2' in line: ndim = 2
                elif '3' in line: ndim = 3

    if claw_pkg is None:
        raise ValueError("*** Could not determine claw_pkg from %s" \
                                % setrun_file)
    else:
        print("claw_pkg = ",claw_pkg)
        
    if ndim is None:
        raise ValueError("*** Could not determine ndim from %s" \
                                % setrun_file)
    else:
        print("ndim = ",ndim)
        

    setrun_module = os.path.splitext(setrun_file)[0]
    exec('from %s import setrun' % setrun_module)

    rundata = setrun()
    c = rundata.clawdata

    limiter_map = {0:'none',1:'minmod',2:'superbee',3:'vanleer',4:'mc'}
    limiter = [limiter_map.get(i,i) for i in c.mthlim]


    bc_map = {0:'user', 1:'extrap', 2:'periodic', 3:'wall'}
    bc_list = ['xlower','xupper']
    if ndim >= 2: bc_list = bc_list + ['ylower','yupper']
    if ndim == 3: bc_list = bc_list + ['zlower','zupper']
    for b in bc_list:
        exec("s = bc_map.get(c.mthbc_%s, c.mthbc_%s)" % (b,b))
        if type(s) is str: 
            exec("""mthbc_%s = "'%s'" """ % (b,s))
        else:
            exec("mthbc_%s = '%s'" % (b,s))
            
    try:
        gauges = c.gauges
    except:
        try:
            gauges = rundata.geodata.gauges
        except:
            gauges = []

    # AMRClaw didn't have regions before, only geoclaw...
    try:
        regions = rundata.geodata.regions
    except:
        regions = []


    # -------------------------------------------------
    # Mapping from old setrun variables to new ones:

    # all packages and dimensions have these parameters:

    mapping = {
        'xlower': c.xlower,
        'xupper': c.xupper,
        'mx': c.mx,
        'num_eqn': c.meqn,
        'num_aux': c.maux,
        'capa_index': c.mcapa,
        't0': c.t0,
        'num_output_times': c.nout,
        'tfinal': c.tfinal,
        'dt_variable': str(c.dt_variable==1),
        'dt_initial': c.dt_initial,
        'dt_max': c.dt_max,
        'cfl_desired': c.cfl_desired,
        'cfl_max': c.cfl_max,
        'steps_max': c.max_steps,
        'order': c.order,
        'transverse_waves': c.order_trans,
        'num_waves': c.mwaves,
        'limiter': str(limiter),
        'source_split': c.src_split,
        'num_ghost': c.mbc,
        'mthbc_xlower': mthbc_xlower,
        'mthbc_xupper': mthbc_xupper,
        'gauges' : gauges}

    # add more for various cases:

    if ndim >= 2:
        mapping['ylower'] = c.ylower
        mapping['yupper'] = c.yupper
        mapping['my'] = c.my
        mapping['mthbc_ylower'] = mthbc_ylower
        mapping['mthbc_yupper'] = mthbc_yupper

    if ndim == 3:
        mapping['zlower'] = c.zlower
        mapping['zupper'] = c.zupper
        mapping['mz'] = c.mz
        mapping['mthbc_zlower'] = mthbc_zlower
        mapping['mthbc_zupper'] = mthbc_zupper

    if claw_pkg in ['amrclaw', 'geoclaw']:
        mapping['amr_levels_max'] =  abs(c.mxnest)
        mapping['refinement_ratios_x'] =  str(c.inratx)
        mapping['refinement_ratios_y'] =  str(c.inraty)
        mapping['refinement_ratios_t'] =  str(c.inratt)
        mapping['aux_type'] =  str(c.auxtype)
        mapping['flag_richardson'] =  str(c.tol > 0.)
        mapping['flag_richardson_tol'] =  abs(c.tol)
        mapping['flag2refine'] =  str(c.tolsp > 0.)
        mapping['flag2refine_tol'] =  c.tolsp
        mapping['regrid_interval'] =  c.kcheck
        mapping['regrid_buffer_width'] =  c.ibuff
        mapping['clustering_cutoff'] =  c.cutoff
        mapping['regions'] =  regions

    if claw_pkg == 'geoclaw':
        try:
            g = rundata.geodata
        except:
            raise ValueError("*** rundata has no geodata attribute")
        mapping['gravity'] =  g.gravity
        mapping['coordinate_system'] =  g.icoordsys
        mapping['earth_radius'] =  g.Rearth
        mapping['coriolis_forcing'] =  (g.icoriolis == 1)
        mapping['sea_level'] =  g.sealevel
        mapping['dry_tolerance'] =  g.drytolerance
        mapping['friction_forcing'] =  (g.ifriction == 1)
        mapping['manning_coefficient'] =  g.coeffmanning
        mapping['friction_depth'] =  g.frictiondepth
        mapping['variable_dt_refinement_ratios'] = g.variable_dt_refinement_ratios
        mapping['wave_tolerance'] =  g.wavetolerance
        mapping['deep_depth'] =  g.depthdeep
        mapping['max_level_deep'] =  g.maxleveldeep
        mapping['topofiles'] =  g.topofiles
        mapping['dtopofiles'] =  g.dtopofiles
        mapping['qinit_type'] =  g.iqinit
        mapping['qinitfiles'] =  g.qinitfiles
        mapping['fixedgrids'] =  g.fixedgrids
        mapping['fgmax_files'] =  []

    if claw_pkg == 'amrclaw' and ndim == 3:
        mapping['refinement_ratios_z'] =  str(c.inratz)

    # Use mapping to convert setrun file:

    if claw_pkg == 'classic' and ndim==1:
        template = open(clawutil \
                 + '/conversion/setrun_template_classic_1d.py').read()
        newtext = template.format(**mapping)

    elif claw_pkg == 'classic' and ndim==2:
        template = open(clawutil \
                 + '/conversion/setrun_template_classic_2d.py').read()
        newtext = template.format(**mapping)

    elif claw_pkg == 'classic' and ndim==3:
        template = open(clawutil \
                 + '/conversion/setrun_template_classic_3d.py').read()
        newtext = template.format(**mapping)

    elif claw_pkg == 'geoclaw' and ndim==2:
        template = open(clawutil \
                 + '/conversion/setrun_template_geoclaw_2d_shallow.py').read()
        newtext = template.format(**mapping)

    elif claw_pkg == 'amrclaw' and ndim==2:
        template = open(clawutil \
                 + '/conversion/setrun_template_amrclaw_2d.py').read()
        newtext = template.format(**mapping)

    else:
        raise ValueError("*** convert not yet implemented " + \
                    "for claw_pkg = %s, ndim = %s" \
                    % (claw_pkg, ndim))

    setrun_text = open(setrun_file).readlines()
    for line in setrun_text:
        if "new_UserData" in line:
            if line.strip()[0] != '#':
                print("*** Warning: call to new_UserData detected...")
                print(line)
                print("*** User data lines from %s have NOT been copied "\
                        % setrun_file)


    os.system("mv %s %s_4.x" % (setrun_file,setrun_file))
    print('Moved %s to %s_4.x ' % (setrun_file,setrun_file))
    open(setrun_file,'w').write(newtext)
    print('===> Created ', setrun_file)
    return claw_pkg, ndim


def copy_Makefile(claw_pkg, ndim):
    if claw_pkg == 'classic' and ndim==1:
        try:
            os.system("mv Makefile Makefile_4.x")
            os.system("cp %s/conversion/Makefile_classic_1d Makefile" \
                    % clawutil)
        except:
            raise Exception("*** Error copying Makefile")
    elif claw_pkg == 'classic' and ndim==2:
        try:
            os.system("mv Makefile Makefile_4.x")
            os.system("cp %s/conversion/Makefile_classic_2d Makefile" \
                    % clawutil)
        except:
            raise Exception("*** Error copying Makefile")
    elif claw_pkg == 'classic' and ndim==3:
        try:
            os.system("mv Makefile Makefile_4.x")
            os.system("cp %s/conversion/Makefile_classic_3d Makefile" \
                    % clawutil)
        except:
            raise Exception("*** Error copying Makefile")
    elif claw_pkg == 'amrclaw' and ndim==2:
        try:
            os.system("mv Makefile Makefile_4.x")
            os.system("cp %s/conversion/Makefile_amrclaw_2d Makefile" \
                    % clawutil)
        except:
            raise Exception("*** Error copying Makefile")
    elif claw_pkg == 'geoclaw' and ndim==2:
        try:
            os.system("mv Makefile Makefile_4.x")
            os.system("cp %s/conversion/Makefile_geoclaw_2d_shallow Makefile" \
                    % clawutil)
        except:
            raise Exception("*** Error copying Makefile")
    else:
        raise ValueError("*** copy_Makefile not yet implemented " + \
                    "for claw_pkg = %s, ndim = %s" \
                    % (claw_pkg, ndim))

    print("Moved Makefile to Makefile_4.x")
    print("===> Created new Makefile template -- must be customized!")
    print("*** Edit Makefile based on Makefile_4.x, e.g. point to")
    print("*** any local files, correct Riemann solver, etc.\n")
    print("*** You might have to make other modifications to local fortran files")


def convert_setplot(setplot_file='setplot.py'):

    setplot_text = open(setplot_file).read()
    setplot_text = setplot_text.replace('pyclaw.plotters','clawpack.visclaw')
    setplot_text = setplot_text.replace('2d_grid','2d_patch')
    setplot_text = setplot_text.replace('gridedges','patchedges')
    setplot_text = setplot_text.replace('gridlines','celledges')
    setplot_text = setplot_text.replace('grid_bgcolor','patch_bgcolor')

    os.system("mv %s %s_4.x" % (setplot_file,setplot_file))
    print('Moved %s to %s_4.x ' % (setplot_file,setplot_file))
    open(setplot_file,'w').write(setplot_text)
    print('===> Created ', setplot_file)

def fix_fortran(claw_pkg, ndim):
    
    import glob
    if claw_pkg=='classic':
        os.system("mv driver.f driver.f_4.x")
        print("Moved driver.f to driver.f_4.x (no longer needed)")

    ffiles = glob.glob('*.f*')

    # Order to permute indices:
    if ndim==1:
        plist = [('q',     [2,1]), ('aux',   [2,1])]
    elif ndim==2:
        plist = [('q',     [3,1,2]), ('aux',   [3,1,2])]
    elif ndim==3:
        plist = [('q',     [4,1,2,3]), ('aux',   [4,1,2,3])]

    for f in ffiles:
        if f[:1] == 'rp':
            print("*** not reordering Riemann solver ",f)
            print("*** switch to library routine from $CLAW/riemann if possible")
        else:
            f4x = f + "_4.x"
            os.system("mv %s %s" % (f,f4x))
            permute_file(f4x, f, plist, write_header=False)
            print("===> Tried to reorder indices in ",f)
            print("Moved original to ",f4x)
    

if __name__ == "__main__":
    claw_pkg, ndim = convert_setrun()
    copy_Makefile(claw_pkg, ndim)
    convert_setplot()

    fix_fortran(claw_pkg, ndim)

    print("*** WARNING -- this only did a first pass at conversion")
    print("*** WARNING -- see hints above for what else needs to be done")
    print("*** WARNING -- and check all files for correctness")




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

from __future__ import absolute_import
import numpy
a = 1.
sigma = 0.5
h0 = 0.1
grav = 9.81
omega = numpy.sqrt(2.*grav*h0) / a 

#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from clawpack.visclaw import colormaps, geoplot

    plotdata.clearfigures()  # clear any old figures,axes,items data

    def set_drytol(current_data):
        # The drytol parameter is used in masking land and water and
        # affects what color map is used for cells with small water depth h.
        # The cell will be plotted as dry if h < drytol.
        # The best value to use often depends on the application and can
        # be set here (measured in meters):
        current_data.user["drytol"] = 1.e-3

    plotdata.beforeframe = set_drytol

    #-----------------------------------------
    # Figure for pcolor plot
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='pcolor', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.title = 'Surface'
    plotaxes.scaled = True

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.surface
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.1
    plotitem.pcolor_cmax = 0.1
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1
    plotaxes.xlimits = [-2,2]
    plotaxes.ylimits = [-2,2]

    # Add contour lines of bathymetry:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    plotitem.contour_levels = linspace(-.1, 0.5, 20)
    plotitem.amr_contour_colors = ['k']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [1]  
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True

    #-----------------------------------------
    # Figure for cross section
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='cross-section', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-2,2]
    plotaxes.ylimits = [-0.15,0.3]
    plotaxes.title = 'Cross section at y=0'
    def plot_topo_xsec(current_data):
        from pylab import plot, cos,sin,where,legend,nan, figure
        t = current_data.t

        x = linspace(-2,2,201)
        y = 0.
        B = h0*(x**2 + y**2)/a**2 - h0
        eta1 = sigma*h0/a**2 * (2.*x*cos(omega*t) + 2.*y*sin(omega*t) -sigma)
        etatrue = where(eta1>B, eta1, nan)
        plot(x, etatrue, 'r', label="true solution", linewidth=2)
        plot(x, B, 'g', label="bathymetry")
        ## plot([0],[-1],'kx',label="Level 1")  # shouldn't show up in plots,
        ## plot([0],[-1],'bo',label="Level 2")  # but will produced desired legend
        plot([0],[-1],'bo',label="Computed")  ## need to fix plotstyle
        legend()
        figure()
    plotaxes.afteraxes = plot_topo_xsec

    plotitem = plotaxes.new_plotitem(plot_type='1d_from_2d_data')

    def xsec(current_data):
        # Return x value and surface eta at this point, along y=0
        from pylab import ravel
        x = current_data.x
        y = current_data.y
        dy = current_data.dy
        q = current_data.q

        ij = numpy.argwhere((y <= dy/2.) & (y > -dy/2.))
        x_slice = ravel(x)[ij]
        eta_slice = ravel(q[3,:,:])[ij]
        return x_slice, eta_slice

    plotitem.map_2d_to_1d = xsec
    plotitem.plotstyle = 'kx'     ## need to be able to set amr_plotstyle
    plotitem.kwargs = {'markersize':3}
    plotitem.amr_show = [1]  # plot on all levels


    #-----------------------------------------
    # Figure for grids alone
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='grids', figno=2)
    plotfigure.show = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-2,2]
    plotaxes.ylimits = [-2,2]
    plotaxes.title = 'grids'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_patch')
    plotitem.amr_patch_bgcolor = ['#ffeeee', '#eeeeff', '#eeffee']
    plotitem.amr_celledges_show = [1,1,0]   
    plotitem.amr_patchedges_show = [1]


    #-----------------------------------------
    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = []             # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

    



"""
Regression tests for 2D advection on a square.
"""

from __future__ import absolute_import
import sys
import unittest

import clawpack.amrclaw.test as test


class Acoustics1DAdjointATest(test.AMRClawRegressionTest):
    r"""Basic test for a 1D acoustics adjoint problem test case"""


    def runTest(self, save=False):

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=0)
        self.check_gauges(save=save, gauge_id=1)
        self.check_gauges(save=save, gauge_id=2)

        # self.check_gauges(save=save, gauge_num=1,
        #                   regression_data_path='regression_data_test2.txt')
        # self.check_gauges(save=save, gauge_num=2,
        #                   regression_data_path='regression_data_test3.txt')

        self.success = True



if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = Acoustics1DAdjointATest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 
    print('Input number of spatial dimensions for this data (default=1):', end=' ')
    ndim = input()
    if ndim == '': 
        ndim=1
    else:
        ndim = int(ndim)

    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for q[0]
    plotfigure = plotdata.new_plotfigure(name='q[0]', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'q0'

    # Set up for item on these axes:
    if ndim==1:
        plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
        plotitem.kwargs = {'linewidth':2,'markersize':5}
        plotitem.plotstyle = '-o'
        plotitem.color = 'b'
    elif ndim==2:
        plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
        from clawpack.visclaw import colormaps
        plotitem.pcolor_cmap = colormaps.yellow_red_blue
        plotitem.add_colorbar = True
    else:
        raise Exception('Default setplot parameters are implemented only for 1D and 2D data')

    plotitem.plot_var = 0
    plotitem.show = True       # show on plot?
    

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 1           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

 



#!/usr/bin/env python
# encoding: utf-8
r"""
Module containing petclaw.geometry.
"""

from __future__ import absolute_import
from clawpack import pyclaw
from clawpack.pyclaw import geometry as pyclaw_geometry

class Patch(pyclaw_geometry.Patch):
    """Parallel Patch class.
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw_geometry.Patch)

    def __init__(self,dimensions):

        super(Patch,self).__init__(dimensions)

        self._da = self._create_DA()
        ranges = self._da.getRanges()
        grid_dimensions = []
        for i,nrange in enumerate(ranges):
            lower = self.lower_global[i] + nrange[0]*self.delta[i]
            upper = self.lower_global[i] + nrange[1]*self.delta[i]
            num_cells   = nrange[1]-nrange[0]

            grid_dimensions.append(pyclaw_geometry.Dimension(lower,upper,
                                   num_cells,name=self.dimensions[i].name))

            if nrange[0] == 0:
                grid_dimensions[-1].on_lower_boundary = True
            else:
                grid_dimensions[-1].on_lower_boundary = False

            if nrange[1] == self.num_cells_global[i]:
                grid_dimensions[-1].on_upper_boundary = True
            else:
                grid_dimensions[-1].on_upper_boundary = False

        self.grid = pyclaw_geometry.Grid(grid_dimensions)

    def _create_DA(self):
        r"""Returns a PETSc DA and associated global Vec.
        Note that no local vector is returned.
        """
        from petsc4py import PETSc

        if hasattr(PETSc.DA, 'PeriodicType'):
            if self.num_dim == 1:
                periodic_type = PETSc.DA.PeriodicType.X
            elif self.num_dim == 2:
                periodic_type = PETSc.DA.PeriodicType.XY
            elif self.num_dim == 3:
                periodic_type = PETSc.DA.PeriodicType.XYZ
            else:
                raise Exception("Invalid number of dimensions")

            DA = PETSc.DA().create(dim=self.num_dim,
                                   dof=1,
                                   sizes=self.num_cells_global,
                                   periodic_type=periodic_type,
                                   stencil_width=0,
                                   comm=PETSc.COMM_WORLD)
        else:
            DA = PETSc.DA().create(dim=self.num_dim,
                                   dof=1,
                                   sizes=self.num_cells_global,
                                   boundary_type=PETSc.DA.BoundaryType.PERIODIC,
                                   stencil_width=0,
                                   comm=PETSc.COMM_WORLD)

        return DA

# ============================================================================
#  PetClaw Domain object definition
# ============================================================================
class Domain(pyclaw_geometry.Domain):
    r""" Parallel Domain Class
    """

    __doc__ += pyclaw.util.add_parent_doc(pyclaw.ClawSolver2D)

    def __init__(self,geom):
        if not isinstance(geom,list):
            geom = [geom]
        if isinstance(geom[0],Patch):
            self.patches = geom
        elif isinstance(geom[0],pyclaw_geometry.Dimension):
            self.patches = [Patch(geom)]

class Dimension(pyclaw_geometry.Dimension):
    def __init__(self, lower, upper, num_cells, name='x',
                 on_lower_boundary=None,on_upper_boundary=None, units=None):
        super(Dimension, self).__init__(lower, upper, num_cells, name,
                                        on_lower_boundary,on_upper_boundary,
                                        units)
